@@ -35,8 +35,4 @@
 GST_DEBUG_CATEGORY_EXTERN (alsa_debug);
 #define GST_CAT_DEFAULT alsa_debug
 
-GstCaps * gst_alsa_probe_supported_formats (GstObject      * obj,
-                                            snd_pcm_t      * handle,
-                                            const GstCaps  * template_caps);
-
 #endif /* __GST_ALSA_H__ */
