@@ -1,5 +1,6 @@
 /* OGG muxer plugin for GStreamer
  * Copyright (C) 2004 Wim Taymans <wim@fluendo.com>
+ * Copyright (C) 2006 Thomas Vander Stichele <thomas at apestaart dot org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,10 +26,11 @@
 #include <gst/base/gstcollectpads.h>
 
 #include <ogg/ogg.h>
-/* memcpy - if someone knows a way to get rid of it, please speak up 
+/* memcpy - if someone knows a way to get rid of it, please speak up
  * note: the ogg docs even say you need this... */
 #include <string.h>
 #include <time.h>
+#include <stdlib.h>             /* rand, srand, atoi */
 
 GST_DEBUG_CATEGORY_STATIC (gst_ogg_mux_debug);
 #define GST_CAT_DEFAULT gst_ogg_mux_debug
@@ -73,8 +75,13 @@ typedef struct
   guint64 duration;             /* duration of current page */
   gboolean eos;
   gint64 offset;
-  GstClockTime timestamp;       /* timestamp for granulepos of last complete
-                                   packet on this page */
+  GstClockTime timestamp;       /* timestamp of the first packet on the next
+                                 * page to be dequeued */
+  GstClockTime timestamp_end;   /* end timestamp of last complete packet on
+                                   the next page to be dequeued */
+  GstClockTime gp_time;         /* time corresponding to the gp value of the
+                                   last complete packet on the next page to be
+                                   dequeued */
 
   GstOggPadState state;         /* state of the pad */
 
@@ -132,7 +139,8 @@ struct _GstOggMuxClass
 };
 
 /* elementfactory information */
-static GstElementDetails gst_ogg_mux_details = GST_ELEMENT_DETAILS ("ogg muxer",
+static const GstElementDetails gst_ogg_mux_details =
+GST_ELEMENT_DETAILS ("Ogg muxer",
     "Codec/Muxer",
     "mux ogg streams (info about ogg: http://xiph.org)",
     "Wim Taymans <wim@fluendo.com>");
@@ -238,7 +246,7 @@ gst_ogg_mux_class_init (GstOggMuxClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = gst_ogg_mux_finalize;
   gobject_class->get_property = gst_ogg_mux_get_property;
@@ -302,7 +310,8 @@ gst_ogg_mux_init (GstOggMux * ogg_mux)
 
   ogg_mux->collect = gst_collect_pads_new ();
   gst_collect_pads_set_function (ogg_mux->collect,
-      (GstCollectPadsFunction) gst_ogg_mux_collected, ogg_mux);
+      (GstCollectPadsFunction) GST_DEBUG_FUNCPTR (gst_ogg_mux_collected),
+      ogg_mux);
 
   gst_ogg_mux_clear (ogg_mux);
 }
@@ -326,15 +335,12 @@ static GstPadLinkReturn
 gst_ogg_mux_sinkconnect (GstPad * pad, GstPad * peer)
 {
   GstOggMux *ogg_mux;
-  gchar *name;
 
   ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
 
-  name = gst_pad_get_name (pad);
+  GST_DEBUG_OBJECT (ogg_mux, "sinkconnect triggered on %s", GST_PAD_NAME (pad));
 
-  GST_DEBUG_OBJECT (ogg_mux, "sinkconnect triggered on %s", name);
-
-  g_free (name);
+  gst_object_unref (ogg_mux);
 
   return GST_PAD_LINK_OK;
 }
@@ -487,10 +493,6 @@ gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page, gboolean delta)
   memcpy (GST_BUFFER_DATA (buffer) + page->header_len,
       page->body, page->body_len);
 
-  /* next_ts was the timestamp of the first buffer put in this page */
-  GST_BUFFER_TIMESTAMP (buffer) = mux->next_ts;
-  GST_BUFFER_OFFSET (buffer) = mux->offset;
-  mux->offset += GST_BUFFER_SIZE (buffer);
   /* Here we set granulepos as our OFFSET_END to give easy direct access to
    * this value later. Before we push it, we reset this to OFFSET + SIZE
    * (see gst_ogg_mux_push_buffer). */
@@ -498,14 +500,25 @@ gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page, gboolean delta)
   if (delta)
     GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
 
+  GST_LOG_OBJECT (mux, GST_GP_FORMAT
+      " created buffer %p from ogg page", ogg_page_granulepos (page));
+
   return buffer;
 }
 
 static GstFlowReturn
 gst_ogg_mux_push_buffer (GstOggMux * mux, GstBuffer * buffer)
 {
-  GST_BUFFER_OFFSET_END (buffer) = GST_BUFFER_OFFSET (buffer) +
-      GST_BUFFER_SIZE (buffer);
+  GstCaps *caps;
+
+  /* fix up OFFSET and OFFSET_END again */
+  GST_BUFFER_OFFSET (buffer) = mux->offset;
+  mux->offset += GST_BUFFER_SIZE (buffer);
+  GST_BUFFER_OFFSET_END (buffer) = mux->offset;
+
+  caps = gst_pad_get_negotiated_caps (mux->srcpad);
+  gst_buffer_set_caps (buffer, caps);
+  gst_caps_unref (caps);
 
   return gst_pad_push (mux->srcpad, buffer);
 }
@@ -533,9 +546,11 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
      * time ordering. */
     if (pad->pagebuffers->length == 0) {
       if (pad->eos) {
-        GST_LOG_OBJECT (pad, "pad is EOS, skipping for dequeue decision");
+        GST_LOG_OBJECT (pad->collect.pad,
+            "pad is EOS, skipping for dequeue decision");
       } else {
-        GST_LOG_OBJECT (pad, "no pages in this queue, can't dequeue");
+        GST_LOG_OBJECT (pad->collect.pad,
+            "no pages in this queue, can't dequeue");
         return FALSE;
       }
     } else {
@@ -553,7 +568,8 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
         }
       }
       if (!valid) {
-        GST_LOG_OBJECT (pad, "No page timestamps in queue, can't dequeue");
+        GST_LOG_OBJECT (pad->collect.pad,
+            "No page timestamps in queue, can't dequeue");
         return FALSE;
       }
     }
@@ -565,11 +581,11 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
   while (walk) {
     GstOggPad *pad = (GstOggPad *) walk->data;
 
-    /* any page with a granulepos of -1 can be pushed immediately. 
+    /* any page with a granulepos of -1 can be pushed immediately.
      * TODO: it CAN be, but it seems silly to do so? */
     buf = g_queue_peek_head (pad->pagebuffers);
     while (buf && GST_BUFFER_OFFSET_END (buf) == -1) {
-      GST_LOG_OBJECT (pad, GST_GP_FORMAT " pushing page", -1);
+      GST_LOG_OBJECT (pad->collect.pad, "[gp        -1] pushing page");
       g_queue_pop_head (pad->pagebuffers);
       *flowret = gst_ogg_mux_push_buffer (mux, buf);
       buf = g_queue_peek_head (pad->pagebuffers);
@@ -579,12 +595,18 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
     if (buf) {
       /* if no oldest buffer yet, take this one */
       if (oldest == GST_CLOCK_TIME_NONE) {
-        oldest = GST_BUFFER_END_TIME (buf);
+        GST_LOG_OBJECT (mux, "no oldest yet, taking buffer %p from pad %"
+            GST_PTR_FORMAT " with gp time %" GST_TIME_FORMAT,
+            buf, pad->collect.pad, GST_TIME_ARGS (GST_BUFFER_OFFSET (buf)));
+        oldest = GST_BUFFER_OFFSET (buf);
         opad = pad;
       } else {
         /* if we have an oldest, compare with this one */
-        if (GST_BUFFER_END_TIME (buf) < oldest) {
-          oldest = GST_BUFFER_END_TIME (buf);
+        if (GST_BUFFER_OFFSET (buf) < oldest) {
+          GST_LOG_OBJECT (mux, "older buffer %p, taking from pad %"
+              GST_PTR_FORMAT " with gp time %" GST_TIME_FORMAT,
+              buf, pad->collect.pad, GST_TIME_ARGS (GST_BUFFER_OFFSET (buf)));
+          oldest = GST_BUFFER_OFFSET (buf);
           opad = pad;
         }
       }
@@ -595,9 +617,10 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
   if (oldest != GST_CLOCK_TIME_NONE) {
     g_assert (opad);
     buf = g_queue_pop_head (opad->pagebuffers);
-    GST_LOG_OBJECT (opad,
-        GST_GP_FORMAT " pushing oldest page (end time %" GST_TIME_FORMAT ")",
-        GST_BUFFER_OFFSET_END (buf), GST_TIME_ARGS (GST_BUFFER_END_TIME (buf)));
+    GST_LOG_OBJECT (opad->collect.pad,
+        GST_GP_FORMAT " pushing oldest page buffer %p (granulepos time %"
+        GST_TIME_FORMAT ")", GST_BUFFER_OFFSET_END (buf), buf,
+        GST_TIME_ARGS (GST_BUFFER_OFFSET (buf)));
     *flowret = gst_ogg_mux_push_buffer (mux, buf);
     ret = TRUE;
   }
@@ -605,21 +628,43 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
   return ret;
 }
 
-/* put the given page on a per-pad queue, timestamping it correctly.
- * after that, dequeue and push as many pages as possible
- * before calling me, make sure that the the pad's timestamp matches
- * the page's granulepos */
+/* put the given ogg page on a per-pad queue, timestamping it correctly.
+ * after that, dequeue and push as many pages as possible.
+ * Caller should make sure:
+ * pad->timestamp     was set with the timestamp of the first packet put
+ *                    on the page
+ * pad->timestamp_end was set with the timestamp + duration of the last packet
+ *                    put on the page
+ * pad->gp_time       was set with the time matching the gp of the last
+ *                    packet put on the page
+ *
+ * will also reset timestamp and timestamp_end, so caller func can restart
+ * counting.
+ */
 static GstFlowReturn
 gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPad * pad, ogg_page * page,
     gboolean delta)
 {
-  GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page, delta);
   GstFlowReturn ret;
+  GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page, delta);
 
-  /* take the timestamp of the last completed packet on this page */
+  /* take the timestamp of the first packet on this page */
   GST_BUFFER_TIMESTAMP (buffer) = pad->timestamp;
+  GST_BUFFER_DURATION (buffer) = pad->timestamp_end - pad->timestamp;
+  /* take the gp time of the last completed packet on this page */
+  GST_BUFFER_OFFSET (buffer) = pad->gp_time;
+
+  /* the next page will start where the current page's end time leaves off */
+  pad->timestamp = pad->timestamp_end;
+
   g_queue_push_tail (pad->pagebuffers, buffer);
-  GST_LOG_OBJECT (pad, GST_GP_FORMAT " queued buffer page (time %" GST_TIME_FORMAT "), %d page buffers queued", ogg_page_granulepos (page), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)), pad->pagebuffers->length);   /* no g_queue_get_length in 2.2 */
+  GST_LOG_OBJECT (pad->collect.pad, GST_GP_FORMAT
+      " queued buffer page %p (gp time %"
+      GST_TIME_FORMAT ", timestamp %" GST_TIME_FORMAT
+      "), %d page buffers queued", ogg_page_granulepos (page),
+      buffer, GST_TIME_ARGS (GST_BUFFER_OFFSET (buffer)),
+      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
+      g_queue_get_length (pad->pagebuffers));
 
   while (gst_ogg_mux_dequeue_page (mux, &ret)) {
     if (ret != GST_FLOW_OK)
@@ -630,43 +675,49 @@ gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPad * pad, ogg_page * page,
 }
 
 /*
- * Given two pads, compare the buffers queued on it and return 0 if they have
- * an equal priority, 1 if the new pad is better, -1 if the old pad is better 
+ * Given two pads, compare the buffers queued on it.
+ * Returns:
+ *  0 if they have an equal priority
+ * -1 if the first is better
+ *  1 if the second is better
+ * Priority decided by: a) validity, b) older timestamp, c) smaller number
+ * of muxed pages
  */
 static gint
-gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * old, GstOggPad * new)
+gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * first,
+    GstOggPad * second)
 {
-  guint64 oldtime, newtime;
+  guint64 firsttime, secondtime;
 
-  /* if the old pad doesn't contain anything or is even NULL, return 
-   * the new pad as best candidate and vice versa */
-  if (old == NULL || old->buffer == NULL)
+  /* if the first pad doesn't contain anything or is even NULL, return
+   * the second pad as best candidate and vice versa */
+  if (first == NULL || first->buffer == NULL)
     return 1;
-  if (new == NULL || new->buffer == NULL)
+  if (second == NULL || second->buffer == NULL)
     return -1;
 
-  /* no timestamp on old buffer, it must go first */
-  oldtime = GST_BUFFER_TIMESTAMP (old->buffer);
-  if (oldtime == GST_CLOCK_TIME_NONE)
+  /* no timestamp on first buffer, it must go first */
+  firsttime = GST_BUFFER_TIMESTAMP (first->buffer);
+  if (firsttime == GST_CLOCK_TIME_NONE)
     return -1;
 
-  /* no timestamp on new buffer, it must go first */
-  newtime = GST_BUFFER_TIMESTAMP (new->buffer);
-  if (newtime == GST_CLOCK_TIME_NONE)
+  /* no timestamp on second buffer, it must go first */
+  secondtime = GST_BUFFER_TIMESTAMP (second->buffer);
+  if (secondtime == GST_CLOCK_TIME_NONE)
     return 1;
 
-  /* old buffer has higher timestamp, new one should go first */
-  if (newtime < oldtime)
+  /* first buffer has higher timestamp, second one should go first */
+  if (secondtime < firsttime)
     return 1;
-  /* new buffer has higher timestamp, old one should go first */
-  else if (newtime > oldtime)
+  /* second buffer has higher timestamp, first one should go first */
+  else if (secondtime > firsttime)
     return -1;
   else {
     /* buffers with equal timestamps, prefer the pad that has the
      * least number of pages muxed */
-    if (new->pageno < old->pageno)
+    if (second->pageno < first->pageno)
       return 1;
-    else if (new->pageno > old->pageno)
+    else if (second->pageno > first->pageno)
       return -1;
   }
 
@@ -694,8 +745,7 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
 
     walk = g_slist_next (walk);
 
-    GST_DEBUG_OBJECT (ogg_mux, "looking at pad %" GST_PTR_FORMAT " (oggpad %p)",
-        data->pad, pad);
+    GST_LOG_OBJECT (data->pad, "looking at pad for buffer");
 
     /* try to get a new buffer for this pad if needed and possible */
     if (pad->buffer == NULL) {
@@ -703,7 +753,7 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
       gboolean incaps;
 
       buf = gst_collect_pads_pop (ogg_mux->collect, data);
-      GST_DEBUG_OBJECT (ogg_mux, "popping buffer %" GST_PTR_FORMAT, buf);
+      GST_LOG_OBJECT (data->pad, "popped buffer %" GST_PTR_FORMAT, buf);
 
       /* On EOS we get a NULL buffer */
       if (buf != NULL) {
@@ -725,8 +775,24 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
           }
         }
       } else {
-        GST_DEBUG_OBJECT (ogg_mux, "EOS on pad");
-        pad->eos = TRUE;
+        GST_DEBUG_OBJECT (data->pad, "EOS on pad");
+        if (!pad->eos) {
+          ogg_page page;
+          GstFlowReturn ret;
+
+          /* Just gone to EOS. Flush existing page(s) */
+          pad->eos = TRUE;
+
+          while (ogg_stream_flush (&pad->stream, &page)) {
+            /* Place page into the per-pad queue */
+            ret = gst_ogg_mux_pad_queue_page (ogg_mux, pad, &page,
+                pad->first_delta);
+            /* increment the page number counter */
+            pad->pageno++;
+            /* mark other pages as delta */
+            pad->first_delta = TRUE;
+          }
+        }
       }
 
       pad->buffer = buf;
@@ -736,14 +802,12 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
      * pull on */
     if (pad->buffer) {
       if (gst_ogg_mux_compare_pads (ogg_mux, bestpad, pad) > 0) {
-        GST_DEBUG_OBJECT (ogg_mux, "best pad now %" GST_PTR_FORMAT
-            " (oggpad %p)", data->pad, pad);
+        GST_LOG_OBJECT (data->pad, "new best pad");
 
         bestpad = pad;
       }
     } else if (!pad->eos) {
-      GST_DEBUG_OBJECT (ogg_mux, "hungry pad %" GST_PTR_FORMAT
-          " (oggpad %p)", data->pad, pad);
+      GST_LOG_OBJECT (data->pad, "hungry pad");
       still_hungry = pad;
     }
   }
@@ -885,7 +949,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     pad->headers = gst_ogg_mux_get_headers (pad);
   }
 
-  GST_LOG_OBJECT (mux, "creating first headers");
+  GST_LOG_OBJECT (mux, "creating BOS pages");
   walk = mux->collect->data;
   while (walk) {
     GstOggPad *pad;
@@ -893,16 +957,20 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     ogg_packet packet;
     ogg_page page;
     GstPad *thepad;
+    GstCaps *caps;
+    GstStructure *structure;
+    GstBuffer *hbuf;
 
     pad = (GstOggPad *) walk->data;
     thepad = pad->collect.pad;
+    caps = gst_pad_get_negotiated_caps (thepad);
+    structure = gst_caps_get_structure (caps, 0);
 
     walk = walk->next;
 
     pad->packetno = 0;
 
-    GST_LOG_OBJECT (mux, "looping over headers for pad %s:%s",
-        GST_DEBUG_PAD_NAME (thepad));
+    GST_LOG_OBJECT (thepad, "looping over headers");
 
     if (pad->headers) {
       buf = GST_BUFFER (pad->headers->data);
@@ -935,13 +1003,25 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     ogg_stream_packetin (&pad->stream, &packet);
     gst_buffer_unref (buf);
 
-    GST_LOG_OBJECT (mux, "flushing page with first packet");
-    while (ogg_stream_flush (&pad->stream, &page)) {
-      GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
+    GST_LOG_OBJECT (thepad, "flushing out BOS page");
+    if (!ogg_stream_flush (&pad->stream, &page))
+      g_critical ("Could not flush BOS page");
 
-      GST_LOG_OBJECT (mux, "swapped out page");
+    hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
+
+    GST_LOG_OBJECT (mux, "swapped out page with mime type %s",
+        gst_structure_get_name (structure));
+
+    /* quick hack: put theora pages at the front.
+     * Ideally, we would have a settable enum for which Ogg
+     * profile we work with, and order based on that */
+    if (strcmp (gst_structure_get_name (structure), "video/x-theora") == 0) {
+      GST_DEBUG_OBJECT (thepad, "putting Theora page at the front");
+      hbufs = g_list_prepend (hbufs, hbuf);
+    } else {
       hbufs = g_list_append (hbufs, hbuf);
     }
+    gst_caps_unref (caps);
   }
 
   GST_LOG_OBJECT (mux, "creating next headers");
@@ -1035,10 +1115,10 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
  *
  * basic idea:
  *
- * 1) find a pad to pull on, this is done by looking at the buffers 
+ * 1) find a pad to pull on, this is done by looking at the buffers
  *    to decide which one to use, we use the 'oldest' one first.
- * 2) store the selected pad and keep on pulling until we fill a 
- *    complete ogg page or the ogg page is filled above the max-delay 
+ * 2) store the selected pad and keep on pulling until we fill a
+ *    complete ogg page or the ogg page is filled above the max-delay
  *    threshold. This is needed because the ogg spec says that
  *    you should fill a complete page with data from the same logical
  *    stream. When the page is filled, go back to 1).
@@ -1061,9 +1141,9 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
   gboolean delta_unit;
   GstFlowReturn ret;
   gint64 granulepos = 0;
-  GstClockTime timestamp;
+  GstClockTime timestamp, gp_time;
 
-  GST_DEBUG_OBJECT (ogg_mux, "collected");
+  GST_LOG_OBJECT (ogg_mux, "collected");
 
   /* queue buffers on all pads; find a buffer with the lowest timestamp */
   best = gst_ogg_mux_queue_pads (ogg_mux);
@@ -1072,17 +1152,18 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
     return GST_FLOW_OK;
   }
 
-  if (best)
-    GST_DEBUG_OBJECT (ogg_mux, "best pad %" GST_PTR_FORMAT " (oggpad %p)"
-        " pulling %" GST_PTR_FORMAT, best->collect.pad, best, ogg_mux->pulling);
-
-  if (!best) {                  /* EOS : FIXME !! We need to handle EOS correctly, and set EOS
-                                   flags on the ogg pages. */
+  if (!best) {
+    /* EOS : FIXME !! We need to handle EOS correctly, and set EOS
+       flags on the ogg pages. */
     GST_DEBUG_OBJECT (ogg_mux, "Pushing EOS");
     gst_pad_push_event (ogg_mux->srcpad, gst_event_new_eos ());
     return GST_FLOW_WRONG_STATE;
   }
 
+  GST_LOG_OBJECT (ogg_mux, "best pad %" GST_PTR_FORMAT
+      ", currently pulling from %" GST_PTR_FORMAT, best->collect.pad,
+      ogg_mux->pulling);
+
   /* if we were already pulling from one pad, but the new "best" buffer is
    * from another pad, we need to check if we have reason to flush a page
    * for the pad we were pulling from before */
@@ -1090,15 +1171,21 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
       ogg_mux->pulling != best && ogg_mux->pulling->buffer) {
     GstOggPad *pad = ogg_mux->pulling;
 
-    GstClockTime last_ts =
-        GST_BUFFER_TIMESTAMP (pad->buffer) + GST_BUFFER_DURATION (pad->buffer);
+    GstClockTime last_ts = GST_BUFFER_END_TIME (pad->buffer);
 
-    /* if the next packet in the current page is going to make the page 
+    /* if the next packet in the current page is going to make the page
      * too long, we need to flush */
     if (last_ts > ogg_mux->next_ts + ogg_mux->max_delay) {
       ogg_page page;
 
+      GST_LOG_OBJECT (pad->collect.pad,
+          GST_GP_FORMAT " stored packet %" G_GINT64_FORMAT
+          " will make page too long, flushing",
+          GST_BUFFER_OFFSET_END (pad->buffer), pad->stream.packetno);
+
       while (ogg_stream_flush (&pad->stream, &page)) {
+        /* end time of this page is the timestamp of the next buffer */
+        ogg_mux->pulling->timestamp_end = GST_BUFFER_TIMESTAMP (pad->buffer);
         /* Place page into the per-pad queue */
         ret = gst_ogg_mux_pad_queue_page (ogg_mux, pad, &page,
             pad->first_delta);
@@ -1112,20 +1199,16 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
     }
   }
 
-  if (ogg_mux->need_headers) {
-    ret = gst_ogg_mux_send_headers (ogg_mux);
-    ogg_mux->need_headers = FALSE;
-  }
-
   /* if we don't know which pad to pull on, use the best one */
   if (ogg_mux->pulling == NULL) {
     ogg_mux->pulling = best;
-    GST_DEBUG_OBJECT (ogg_mux, "pulling now %" GST_PTR_FORMAT " (oggpad %p)",
-        ogg_mux->pulling->collect.pad, ogg_mux->pulling);
+    GST_LOG_OBJECT (ogg_mux->pulling->collect.pad, "pulling from best pad");
 
-    /* remember timestamp of first buffer for this new pad */
+    /* remember timestamp and gp time of first buffer for this new pad */
     if (ogg_mux->pulling != NULL) {
       ogg_mux->next_ts = GST_BUFFER_TIMESTAMP (ogg_mux->pulling->buffer);
+      GST_LOG_OBJECT (ogg_mux->pulling->collect.pad, "updated times, next ts %"
+          GST_TIME_FORMAT, GST_TIME_ARGS (ogg_mux->next_ts));
     } else {
       /* no pad to pull on, send EOS */
       gst_pad_push_event (ogg_mux->srcpad, gst_event_new_eos ());
@@ -1133,6 +1216,11 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
     }
   }
 
+  if (ogg_mux->need_headers) {
+    ret = gst_ogg_mux_send_headers (ogg_mux);
+    ogg_mux->need_headers = FALSE;
+  }
+
   /* we are pulling from a pad, continue to do so until a page
    * has been filled and queued */
   if (ogg_mux->pulling != NULL) {
@@ -1143,8 +1231,7 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
     gint64 duration;
     gboolean force_flush;
 
-    GST_DEBUG_OBJECT (ogg_mux, "pulling now %" GST_PTR_FORMAT " (oggpad %p)",
-        ogg_mux->pulling->collect.pad, ogg_mux->pulling);
+    GST_LOG_OBJECT (ogg_mux->pulling->collect.pad, "pulling from pad");
 
     /* now see if we have a buffer */
     buf = pad->buffer;
@@ -1157,6 +1244,14 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
     delta_unit = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
     duration = GST_BUFFER_DURATION (buf);
 
+    /* if the current "next timestamp" on the pad is unset, then this is the
+     * first packet on the new page.  Update our pad's page timestamp */
+    if (ogg_mux->pulling->timestamp == GST_CLOCK_TIME_NONE) {
+      ogg_mux->pulling->timestamp = GST_BUFFER_TIMESTAMP (buf);
+      GST_LOG_OBJECT (ogg_mux->pulling->collect.pad,
+          "updated pad timestamp to %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
+    }
     /* create a packet from the buffer */
     packet.packet = GST_BUFFER_DATA (buf);
     packet.bytes = GST_BUFFER_SIZE (buf);
@@ -1166,6 +1261,9 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
     /* mark BOS and packet number */
     packet.b_o_s = (pad->packetno == 0);
     packet.packetno = pad->packetno++;
+    GST_LOG_OBJECT (pad->collect.pad, GST_GP_FORMAT
+        " packet %" G_GINT64_FORMAT " (%ld bytes) created from buffer",
+        packet.granulepos, packet.packetno, packet.bytes);
 
     packet.e_o_s = 0;
     tmpbuf = NULL;
@@ -1181,11 +1279,17 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
       }
     }
 
-    /* flush the currently built page if neccesary */
+    /* flush the currently built page if necessary */
     if (force_flush) {
+      GST_LOG_OBJECT (pad->collect.pad,
+          GST_GP_FORMAT " forced flush of page before this packet",
+          GST_BUFFER_OFFSET_END (pad->buffer));
       while (ogg_stream_flush (&pad->stream, &page)) {
+        /* end time of this page is the timestamp of the next buffer */
+        ogg_mux->pulling->timestamp_end = GST_BUFFER_TIMESTAMP (pad->buffer);
         ret = gst_ogg_mux_pad_queue_page (ogg_mux, pad, &page,
             pad->first_delta);
+
         /* increment the page number counter */
         pad->pageno++;
         /* mark other pages as delta */
@@ -1226,34 +1330,61 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
 
     /* swap the packet in */
     if (packet.e_o_s == 1)
-      GST_DEBUG_OBJECT (pad, "swapping in EOS packet");
+      GST_DEBUG_OBJECT (pad->collect.pad, "swapping in EOS packet");
     if (packet.b_o_s == 1)
-      GST_DEBUG_OBJECT (pad, "swapping in BOS packet");
+      GST_DEBUG_OBJECT (pad->collect.pad, "swapping in BOS packet");
 
     ogg_stream_packetin (&pad->stream, &packet);
 
+    gp_time = GST_BUFFER_OFFSET (pad->buffer);
     granulepos = GST_BUFFER_OFFSET_END (pad->buffer);
     timestamp = GST_BUFFER_TIMESTAMP (pad->buffer);
 
+    GST_LOG_OBJECT (pad->collect.pad,
+        GST_GP_FORMAT " packet %" G_GINT64_FORMAT ", gp time %"
+        GST_TIME_FORMAT ", timestamp %" GST_TIME_FORMAT " packetin'd",
+        granulepos, packet.packetno, GST_TIME_ARGS (gp_time),
+        GST_TIME_ARGS (timestamp));
     /* don't need the old buffer anymore */
     gst_buffer_unref (pad->buffer);
     /* store new readahead buffer */
     pad->buffer = tmpbuf;
 
-    /* let ogg write out the pages now. The packet we got could end 
+    /* let ogg write out the pages now. The packet we got could end
      * up in more than one page so we need to write them all */
     if (ogg_stream_pageout (&pad->stream, &page) > 0) {
+      /* we have a new page, so we need to timestamp it correctly.
+       * if this fresh packet ends on this page, then the page's granulepos
+       * comes from that packet, and we should set this buffer's timestamp */
+
+      GST_LOG_OBJECT (pad->collect.pad,
+          GST_GP_FORMAT " packet %" G_GINT64_FORMAT ", time %"
+          GST_TIME_FORMAT ") caused new page",
+          granulepos, packet.packetno, GST_TIME_ARGS (timestamp));
+      GST_LOG_OBJECT (pad->collect.pad,
+          GST_GP_FORMAT " new page %d", ogg_page_granulepos (&page),
+          pad->stream.pageno);
+
       if (ogg_page_granulepos (&page) == granulepos) {
-        /* the packet we streamed in finishes on the page,
+        /* the packet we streamed in finishes on the current page,
          * because the page's granulepos is the granulepos of the last
          * packet completed on that page,
          * so update the timestamp that we will give to the page */
-        pad->timestamp = timestamp;
-        GST_DEBUG_OBJECT (pad, "Timestamp of pad is %" GST_TIME_FORMAT
-            ", granulepos is %lld", GST_TIME_ARGS (timestamp), granulepos);
+        GST_LOG_OBJECT (pad->collect.pad,
+            GST_GP_FORMAT
+            " packet finishes on current page, updating gp time to %"
+            GST_TIME_FORMAT, granulepos, GST_TIME_ARGS (gp_time));
+        pad->gp_time = gp_time;
+      } else {
+        GST_LOG_OBJECT (pad->collect.pad,
+            GST_GP_FORMAT
+            " packet spans beyond current page, keeping old gp time %"
+            GST_TIME_FORMAT, granulepos, GST_TIME_ARGS (pad->gp_time));
       }
 
       /* push the page */
+      /* end time of this page is the timestamp of the next buffer */
+      pad->timestamp_end = timestamp;
       ret = gst_ogg_mux_pad_queue_page (ogg_mux, pad, &page, pad->first_delta);
       pad->pageno++;
       /* mark next pages as delta */
@@ -1264,12 +1395,12 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
       while (ogg_stream_pageout (&pad->stream, &page) > 0) {
         if (ogg_page_granulepos (&page) == granulepos) {
           /* the page has taken up the new packet completely, which means
-           * the packet ends the page and we can update the timestamp
+           * the packet ends the page and we can update the gp time
            * before pushing out */
-          pad->timestamp = timestamp;
+          pad->gp_time = gp_time;
         }
 
-        /* we have a complete page now, we can push the page 
+        /* we have a complete page now, we can push the page
          * and make sure to pull on a new pad the next time around */
         ret = gst_ogg_mux_pad_queue_page (ogg_mux, pad, &page,
             pad->first_delta);
@@ -1279,19 +1410,19 @@ gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
       /* need a new page as well */
       pad->new_page = TRUE;
       pad->duration = 0;
-      /* we're done pulling on this pad, make sure to choose a new 
+      /* we're done pulling on this pad, make sure to choose a new
        * pad for pulling in the next iteration */
       ogg_mux->pulling = NULL;
     }
 
-    /* Update the timestamp, if neccesary, since and future page will have at
-     * least this timestamp.
+    /* Update the gp time, if necessary, since any future page will have at
+     * least this gp time.
      */
-    if (pad->timestamp < timestamp) {
-      pad->timestamp = timestamp;
-      GST_DEBUG_OBJECT (ogg_mux, "Updated timestamp of pad %" GST_PTR_FORMAT
-          " (oggpad %p) to %" GST_TIME_FORMAT, pad->collect.pad, pad,
-          GST_TIME_ARGS (timestamp));
+    if (pad->gp_time < gp_time) {
+      pad->gp_time = gp_time;
+      GST_LOG_OBJECT (pad->collect.pad,
+          "Updated running gp time of pad %" GST_PTR_FORMAT
+          " to %" GST_TIME_FORMAT, pad->collect.pad, GST_TIME_ARGS (gp_time));
     }
   }
 
