@@ -139,6 +139,7 @@ gst_base_audio_src_init (GstBaseAudioSrc * baseaudiosrc,
   gst_pad_set_fixatecaps_function (GST_BASE_SRC_PAD (baseaudiosrc),
       gst_base_audio_src_fixate);
 
+  gst_base_src_set_live (GST_BASE_SRC (baseaudiosrc), TRUE);
   gst_base_src_set_format (GST_BASE_SRC (baseaudiosrc), GST_FORMAT_TIME);
 }
 
@@ -417,6 +418,7 @@ gst_base_audio_src_change_state (GstElement * element,
       gst_ring_buffer_set_flushing (src->ringbuffer, FALSE);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
+      gst_ring_buffer_may_start (src->ringbuffer, TRUE);
       break;
     default:
       break;
@@ -426,6 +428,7 @@ gst_base_audio_src_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      gst_ring_buffer_may_start (src->ringbuffer, FALSE);
       gst_ring_buffer_pause (src->ringbuffer);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
@@ -435,7 +438,7 @@ gst_base_audio_src_change_state (GstElement * element,
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       gst_ring_buffer_close_device (src->ringbuffer);
-      gst_object_unref (src->ringbuffer);
+      gst_object_unparent (GST_OBJECT_CAST (src->ringbuffer));
       src->ringbuffer = NULL;
       break;
     default:
