@@ -38,7 +38,7 @@
 /**
  * SECTION:gstcddabasesrc
  * @short_description: Base class for CD digital audio (CDDA) sources
- * 
+ *
  * <refsect2>
  * <para>
  * Provides a base class for CDDA sources, which handles things like seeking,
@@ -78,6 +78,7 @@
 #endif
 
 #include <string.h>
+#include <stdlib.h>             /* for strtol */
 
 #include "gstcddabasesrc.h"
 #include "gst/gst-i18n-plugin.h"
@@ -155,7 +156,7 @@ static GType
 gst_cdda_base_src_mode_get_type (void)
 {
   static GType mode_type;       /* 0 */
-  static GEnumValue modes[] = {
+  static const GEnumValue modes[] = {
     {GST_CDDA_BASE_SRC_MODE_NORMAL, "Stream consists of a single track",
         "normal"},
     {GST_CDDA_BASE_SRC_MODE_CONTINUOUS, "Stream consists of the whole disc",
