@@ -56,16 +56,35 @@ gst_riff_create_video_caps (guint32 codec_fcc,
   GstBuffer *palette = NULL;
 
   switch (codec_fcc) {
-    case GST_MAKE_FOURCC ('D', 'I', 'B', ' '):
-      caps = gst_caps_new_simple ("video/x-raw-rgb",
-          "bpp", G_TYPE_INT, 8,
-          "depth", G_TYPE_INT, 8, "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
+    case GST_MAKE_FOURCC ('D', 'I', 'B', ' '):{
+      gint bpp = (strf && strf->bit_cnt != 0) ? strf->bit_cnt : 8;
+
+      if (strf) {
+        if (bpp == 8) {
+          caps = gst_caps_new_simple ("video/x-raw-rgb",
+              "bpp", G_TYPE_INT, 8, "depth", G_TYPE_INT, 8,
+              "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
+        } else if (bpp == 24) {
+          caps = gst_caps_new_simple ("video/x-raw-rgb",
+              "bpp", G_TYPE_INT, 24, "depth", G_TYPE_INT, 24,
+              "endianness", G_TYPE_INT, G_BIG_ENDIAN,
+              "red_mask", G_TYPE_INT, 0xff, "green_mask", G_TYPE_INT, 0xff00,
+              "blue_mask", G_TYPE_INT, 0xff0000, NULL);
+        } else {
+          GST_WARNING ("Unhandled DIB RGB depth: %d", bpp);
+        }
+      } else {
+        /* for template */
+        caps = gst_caps_from_string ("video/x-raw-rgb, bpp = (int) { 8, 24 }, "
+            "depth = (int) { 8, 24}");
+      }
+
       palette = strf_data;
       strf_data = NULL;
       if (codec_name)
-        *codec_name = g_strdup ("Palettized 8-bit RGB");
+        *codec_name = g_strdup_printf ("Palettized %d-bit RGB", bpp);
       break;
-
+    }
     case GST_MAKE_FOURCC ('I', '4', '2', '0'):
       caps = gst_caps_new_simple ("video/x-raw-yuv",
           "format", GST_TYPE_FOURCC, codec_fcc, NULL);
@@ -80,6 +99,13 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Uncompressed packed YUV 4:2:2");
       break;
 
+    case GST_MAKE_FOURCC ('Y', 'V', 'U', '9'):
+      caps = gst_caps_new_simple ("video/x-raw-yuv",
+          "format", GST_TYPE_FOURCC, codec_fcc, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Uncompressed packed YVU 4:1:0");
+      break;
+
     case GST_MAKE_FOURCC ('M', 'J', 'P', 'G'): /* YUY2 MJPEG */
     case GST_MAKE_FOURCC ('A', 'V', 'R', 'n'):
     case GST_MAKE_FOURCC ('I', 'J', 'P', 'G'):
@@ -191,6 +217,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Intel H.263");
       break;
 
+    case GST_MAKE_FOURCC ('H', '2', '6', '4'):
     case GST_MAKE_FOURCC ('h', '2', '6', '4'):
       caps = gst_caps_new_simple ("video/x-h264", NULL);
       if (codec_name)
@@ -203,6 +230,13 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("VideoSoft H.264");
       break;
 
+    case GST_MAKE_FOURCC ('S', 'E', 'D', 'G'):
+      caps = gst_caps_new_simple ("video/mpeg",
+          "mpegversion", G_TYPE_INT, 4, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Samsung MPEG-4");
+      break;
+
     case GST_MAKE_FOURCC ('D', 'I', 'V', '3'):
     case GST_MAKE_FOURCC ('d', 'i', 'v', '3'):
     case GST_MAKE_FOURCC ('D', 'I', 'V', '4'):
@@ -516,6 +550,13 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Flash Video 1");
       break;
 
+    case GST_MAKE_FOURCC ('V', 'M', 'n', 'c'):
+      caps = gst_caps_new_simple ("video/x-vmnc",
+          "version", G_TYPE_INT, 1, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("VMWare NC Video");
+      break;
+
     default:
       GST_WARNING ("Unknown video fourcc %" GST_FOURCC_FORMAT,
           GST_FOURCC_ARGS (codec_fcc));
@@ -558,7 +599,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
 
     if (GST_BUFFER_SIZE (palette) >= (num_colors * 4)) {
       /* palette is always at least 256*4 bytes */
-      copy = gst_buffer_new_and_alloc (MAX (num_colors * 4, 256 * 4));
+      copy =
+          gst_buffer_new_and_alloc (MAX (GST_BUFFER_SIZE (palette), 256 * 4));
       memcpy (GST_BUFFER_DATA (copy), GST_BUFFER_DATA (palette),
           GST_BUFFER_SIZE (palette));
 
@@ -844,9 +886,9 @@ gst_riff_create_audio_caps (guint16 codec_id,
       guint32 subformat_guid[4];
       const guint8 *data;
 
-      if (GST_BUFFER_SIZE (strf_data) != 22) {
+      if (strf_data == NULL || GST_BUFFER_SIZE (strf_data) != 22) {
         GST_WARNING ("WAVE_FORMAT_EXTENSIBLE data size is %d (expected: 22)",
-            GST_BUFFER_SIZE (strf_data));
+            (strf_data) ? GST_BUFFER_SIZE (strf_data) : -1);
         return NULL;
       }
 
@@ -978,6 +1020,7 @@ gst_riff_create_video_template_caps (void)
   guint32 tags[] = {
     GST_MAKE_FOURCC ('I', '4', '2', '0'),
     GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'),
+    GST_MAKE_FOURCC ('Y', 'V', 'U', '9'),
     GST_MAKE_FOURCC ('M', 'J', 'P', 'G'),
     GST_MAKE_FOURCC ('D', 'V', 'S', 'D'),
     GST_MAKE_FOURCC ('W', 'M', 'V', '1'),
@@ -1022,6 +1065,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('P', 'N', 'G', ' '),
     GST_MAKE_FOURCC ('C', 'Y', 'U', 'V'),
     GST_MAKE_FOURCC ('F', 'L', 'V', '1'),
+    GST_MAKE_FOURCC ('V', 'M', 'n', 'c'),
     /* FILL ME */
     0
   };
