@@ -30,8 +30,13 @@
 GST_DEBUG_CATEGORY_STATIC (gst_play_base_bin_debug);
 #define GST_CAT_DEFAULT gst_play_base_bin_debug
 
+#if 0
+#define DEFAULT_QUEUE_THRESHOLD ((2 * GST_SECOND) / 3)
+#define DEFAULT_QUEUE_SIZE  (GST_SECOND / 2)
+#else
 #define DEFAULT_QUEUE_THRESHOLD (2 * GST_SECOND)
 #define DEFAULT_QUEUE_SIZE  (3 * GST_SECOND)
+#endif
 
 #define GROUP_LOCK(pbb) g_mutex_lock (pbb->group_lock)
 #define GROUP_UNLOCK(pbb) g_mutex_unlock (pbb->group_lock)
@@ -62,6 +67,7 @@ enum
   ARG_QUEUE_THRESHOLD,
   ARG_NSTREAMS,
   ARG_STREAMINFO,
+  ARG_STREAMINFO_VALUES,
   ARG_SOURCE,
   ARG_VIDEO,
   ARG_AUDIO,
@@ -80,6 +86,8 @@ static void gst_play_base_bin_get_property (GObject * object, guint prop_id,
 static GstStateChangeReturn gst_play_base_bin_change_state (GstElement *
     element, GstStateChange transition);
 const GList *gst_play_base_bin_get_streaminfo (GstPlayBaseBin * play_base_bin);
+const GValueArray *gst_play_base_bin_get_streaminfo_value_array (GstPlayBaseBin
+    * play_base_bin);
 
 static gboolean prepare_output (GstPlayBaseBin * play_base_bin);
 static void set_active_source (GstPlayBaseBin * play_base_bin,
@@ -131,7 +139,7 @@ gst_play_base_bin_class_init (GstPlayBaseBinClass * klass)
   gstelement_klass = (GstElementClass *) klass;
   gstbin_klass = (GstBinClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_PIPELINE);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_klass->set_property = gst_play_base_bin_set_property;
   gobject_klass->get_property = gst_play_base_bin_get_property;
@@ -158,6 +166,11 @@ gst_play_base_bin_class_init (GstPlayBaseBinClass * klass)
   g_object_class_install_property (gobject_klass, ARG_STREAMINFO,
       g_param_spec_pointer ("stream-info", "Stream info", "List of streaminfo",
           G_PARAM_READABLE));
+  g_object_class_install_property (gobject_klass, ARG_STREAMINFO_VALUES,
+      g_param_spec_value_array ("stream-info-value-array",
+          "StreamInfo GValueArray", "value array of streaminfo",
+          g_param_spec_object ("streaminfo", "StreamInfo", "Streaminfo object",
+              GST_TYPE_STREAM_INFO, G_PARAM_READABLE), G_PARAM_READABLE));
   g_object_class_install_property (gobject_klass, ARG_SOURCE,
       g_param_spec_object ("source", "Source", "Source element",
           GST_TYPE_ELEMENT, G_PARAM_READABLE));
@@ -242,6 +255,7 @@ group_create (GstPlayBaseBin * play_base_bin)
 
   group = g_new0 (GstPlayBaseGroup, 1);
   group->bin = play_base_bin;
+  group->streaminfo_value_array = g_value_array_new (0);
 
   return group;
 }
@@ -333,21 +347,15 @@ group_destroy (GstPlayBaseGroup * group)
       }
     }
 
-    /* if the group is currently being played, we have to remove the element 
+    /* if the group is currently being played, we have to remove the element
      * from the thread */
-    if (get_active_group (play_base_bin) == group) {
-      GST_LOG ("removing preroll element %s", GST_ELEMENT_NAME (element));
-      gst_element_set_state (element, GST_STATE_NULL);
-      gst_element_set_state (group->type[n].selector, GST_STATE_NULL);
-      gst_bin_remove (group->type[n].bin, element);
-      gst_bin_remove (group->type[n].bin, group->type[n].selector);
-    } else {
-      /* else we can just unref it */
-      gst_element_set_state (element, GST_STATE_NULL);
-      gst_element_set_state (group->type[n].selector, GST_STATE_NULL);
-      gst_object_unref (element);
-      gst_object_unref (group->type[n].selector);
-    }
+    gst_element_set_state (element, GST_STATE_NULL);
+    gst_element_set_state (group->type[n].selector, GST_STATE_NULL);
+
+    GST_LOG ("removing preroll element %s", GST_ELEMENT_NAME (element));
+
+    gst_bin_remove (group->type[n].bin, element);
+    gst_bin_remove (group->type[n].bin, group->type[n].selector);
 
     group->type[n].preroll = NULL;
     group->type[n].selector = NULL;
@@ -357,11 +365,12 @@ group_destroy (GstPlayBaseGroup * group)
   /* free the streaminfo too */
   g_list_foreach (group->streaminfo, (GFunc) g_object_unref, NULL);
   g_list_free (group->streaminfo);
+  g_value_array_free (group->streaminfo_value_array);
   g_free (group);
 }
 
 /*
- * is called when the current building group is completely finished 
+ * is called when the current building group is completely finished
  * and ready for playback
  *
  * This function grabs lock, so take care when calling.
@@ -408,8 +417,10 @@ group_commit (GstPlayBaseBin * play_base_bin, gboolean fatal, gboolean subtitle)
       sig_id =
           GPOINTER_TO_INT (g_object_get_data (G_OBJECT (element), "signal_id"));
 
-      GST_LOG ("removing preroll signal %s", GST_ELEMENT_NAME (element));
-      g_signal_handler_disconnect (G_OBJECT (element), sig_id);
+      if (sig_id) {
+        GST_LOG ("removing preroll signal %s", GST_ELEMENT_NAME (element));
+        g_signal_handler_disconnect (G_OBJECT (element), sig_id);
+      }
     }
   }
 
@@ -495,16 +506,50 @@ check_queue (GstPad * pad, GstBuffer * data, gpointer user_data)
 
 /* this signal will be fired when one of the queues with raw
  * data is filled. This means that the group building stage is over 
- * and playback of the new queued group should start */
+ * and playback of the new queued group should start
+ *
+ * If this queue overruns we can potentially create a deadlock when:
+ *
+ *  1) the max-bytes is hit and
+ *  2) the min-time is not hit.
+ *
+ * We recover from this situation in the overrun callback by
+ * setting the max-bytes to unlimited if we see that there is
+ * a current-time-level (which means some sort of timestamping is
+ * done).
+ */
 static void
-queue_overrun (GstElement * element, GstPlayBaseBin * play_base_bin)
+queue_overrun (GstElement * queue, GstPlayBaseBin * play_base_bin)
 {
-  GST_DEBUG ("queue %s overrun", GST_ELEMENT_NAME (element));
+  GST_DEBUG ("queue %s overrun", GST_ELEMENT_NAME (queue));
 
-  group_commit (play_base_bin, FALSE, FALSE);
+  /* if we're streaming, check if we had a deadlock with the 
+   * max-bytes <> min-time thresholds */
+  if (play_base_bin->is_stream) {
+    guint64 time, min_time;
 
-  g_signal_handlers_disconnect_by_func (element,
+    g_object_get (G_OBJECT (queue), "current-level-time", &time,
+        "min-threshold-time", &min_time, NULL);
+
+    GST_DEBUG_OBJECT (play_base_bin, "streaming mode, queue %s current %"
+        GST_TIME_FORMAT ", min %" GST_TIME_FORMAT,
+        GST_ELEMENT_NAME (queue),
+        GST_TIME_ARGS (time), GST_TIME_ARGS (min_time));
+    if (time != 0) {
+      /* queue knows about time, disable bytes checking. */
+      g_object_set (G_OBJECT (queue), "max-size-bytes", 0, NULL);
+    }
+  }
+
+  group_commit (play_base_bin, FALSE,
+      GST_OBJECT_PARENT (GST_OBJECT_CAST (queue)) ==
+      GST_OBJECT (play_base_bin->subtitle));
+
+  g_signal_handlers_disconnect_by_func (queue,
       G_CALLBACK (queue_overrun), play_base_bin);
+  /* We have disconnected this signal, remove the signal_id from the object
+     data */
+  g_object_set_data (G_OBJECT (queue), "signal_id", NULL);
 }
 
 /* Used for time-based buffering. */
@@ -516,6 +561,8 @@ queue_threshold_reached (GstElement * queue, GstPlayBaseBin * play_base_bin)
   GST_DEBUG ("Running");
 
   /* play */
+  g_signal_handlers_disconnect_by_func (queue,
+      G_CALLBACK (queue_threshold_reached), play_base_bin);
   g_object_set (queue, "min-threshold-time", (guint64) 0, NULL);
 
   data = g_object_get_data (G_OBJECT (queue), "probe");
@@ -536,14 +583,41 @@ queue_threshold_reached (GstElement * queue, GstPlayBaseBin * play_base_bin)
   }
 }
 
+/* this signal is only added when in streaming mode to catch underruns
+ */
 static void
 queue_out_of_data (GstElement * queue, GstPlayBaseBin * play_base_bin)
 {
+  guint64 time, min_time;
+  guint bytes, max_bytes;
+
   GST_DEBUG ("Underrun, re-caching");
 
+  g_object_get (G_OBJECT (queue), "current-level-time", &time,
+      "current-level-bytes", &bytes,
+      "max-size-bytes", &max_bytes, "min-threshold-time", &min_time, NULL);
+
+  GST_DEBUG_OBJECT (play_base_bin, "streaming mode, queue %s current %"
+      GST_TIME_FORMAT ", min %" GST_TIME_FORMAT
+      ", bytes %d",
+      GST_ELEMENT_NAME (queue),
+      GST_TIME_ARGS (time), GST_TIME_ARGS (min_time), bytes);
+
+  /* if the bytes in the queue represent time, we disable bytes
+   * overrun checking to not cause deadlocks.
+   */
+  if (bytes && time != 0 && time < min_time) {
+    /* queue knows about time but is filled with bytes that do
+     * not represent min-threshold time, disable bytes checking so
+     * the queue can grow some more. */
+    g_object_set (G_OBJECT (queue), "max-size-bytes", 0, NULL);
+  }
+
   /* On underrun, we want to temoprarily pause playback, set a "min-size"
    * threshold and wait for the running signal and then play again. Take
    * care of possible deadlocks and so on, */
+  g_signal_connect (G_OBJECT (queue), "running",
+      G_CALLBACK (queue_threshold_reached), play_base_bin);
   g_object_set (queue, "min-threshold-time",
       (guint64) play_base_bin->queue_threshold, NULL);
 
@@ -605,6 +679,19 @@ gen_preroll_element (GstPlayBaseBin * play_base_bin,
   g_free (name);
   g_free (padname);
 
+  /* for buffering of raw data we ideally want to buffer a
+   * very small amount of buffers since the memory used by
+   * this raw data can be enormously huge.
+   *
+   * FIXME: we abuse this buffer to do network buffering since
+   * we can then easily do time-based buffering. The better
+   * solution would be to add a specific network queue right
+   * after the source that measures the datarate and scales this
+   * queue of encoded data instead.
+   *
+   * We use an upper limit of typically a few seconds here but
+   * cap in case no timestamps are set on the raw data (bad!).
+   */
   g_object_set (G_OBJECT (preroll),
       "max-size-buffers", 0, "max-size-bytes",
       ((type == GST_STREAM_TYPE_VIDEO) ? 25 : 1) * 1024 * 1024,
@@ -635,9 +722,22 @@ gen_preroll_element (GstPlayBaseBin * play_base_bin,
     g_object_set_data (G_OBJECT (preroll), "pbb", play_base_bin);
     g_object_set_data (G_OBJECT (preroll), "probe", GINT_TO_POINTER (id));
 
+    /* 
+     * When we connect this queue, it will start running and immediatly
+     * fire an underrun when:
+     *
+     * 1) the max-bytes is hit
+     * 2) the min-time is not hit.
+     *
+     * We recover from this situation in the out_of_data callback by
+     * setting the max-bytes to unlimited if we see that there is
+     * a current-time-level (which means some sort of timestamping is
+     * done).
+     */
     g_signal_connect (G_OBJECT (preroll), "underrun",
         G_CALLBACK (queue_out_of_data), play_base_bin);
   }
+
   /* keep a ref to the signal id so that we can disconnect the signal callback
    * when we are done with the preroll */
   g_object_set_data (G_OBJECT (preroll), "signal_id", GINT_TO_POINTER (sig));
@@ -702,19 +802,23 @@ remove_groups (GstPlayBaseBin * play_base_bin)
   GROUP_UNLOCK (play_base_bin);
 }
 
-/* 
- * Add/remove a single stream to current  building group.
+/*
+ * Add/remove a single stream to current building group.
  *
  * Must be called with group-lock held.
  */
 static void
 add_stream (GstPlayBaseGroup * group, GstStreamInfo * info)
 {
+  GValue v = { 0, };
   GST_DEBUG ("add stream to group %p", group);
 
   /* keep ref to the group */
   g_object_set_data (G_OBJECT (info), "group", group);
 
+  g_value_init (&v, G_TYPE_OBJECT);
+  g_value_set_object (&v, info);
+  g_value_array_append (group->streaminfo_value_array, &v);
   group->streaminfo = g_list_append (group->streaminfo, info);
   if (info->type > 0 && info->type <= NUM_TYPES) {
     group->type[info->type - 1].npads++;
@@ -921,7 +1025,8 @@ mute_stream (GstPad * pad, GstBuffer * buf, gpointer data)
   g_object_set (G_OBJECT (info), "mute", TRUE, NULL);
   id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (info), "mute_probe"));
   g_object_set_data (G_OBJECT (info), "mute_probe", NULL);
-  gst_pad_remove_buffer_probe (GST_PAD_CAST (info->object), id);
+  if (id > 0)
+    gst_pad_remove_buffer_probe (GST_PAD_CAST (info->object), id);
 
   /* no data */
   return FALSE;
@@ -1057,7 +1162,7 @@ setup_subtitle (GstPlayBaseBin * play_base_bin, gchar * sub_uri)
     return NULL;
   subparse = gst_element_factory_make ("decodebin", "subtitle-decoder");
 
-  subbin = gst_bin_new ("subbin");
+  subbin = gst_bin_new ("subtitle-bin");
   gst_bin_add_many (GST_BIN (subbin), source, subparse, NULL);
   gst_element_link (source, subparse);
 
@@ -1096,6 +1201,14 @@ gen_source_element (GstPlayBaseBin * play_base_bin, GstElement ** subbin)
       !strncmp (play_base_bin->uri, "rtp://", 6) ||
       !strncmp (play_base_bin->uri, "rtsp://", 7);
 
+  /* make HTTP sources send extra headers so we get icecast
+   * metadata in case the stream is an icecast stream */
+  if (!strncmp (play_base_bin->uri, "http://", 7) &&
+      g_object_class_find_property (G_OBJECT_GET_CLASS (source),
+          "iradio-mode")) {
+    g_object_set (source, "iradio-mode", TRUE, NULL);
+  }
+
   return source;
 }
 
@@ -1130,9 +1243,12 @@ setup_substreams (GstPlayBaseBin * play_base_bin)
     if (info->type == GST_STREAM_TYPE_UNKNOWN) {
       guint id;
 
-      id = gst_pad_add_buffer_probe (GST_PAD_CAST (info->object),
-          G_CALLBACK (mute_stream), info);
-      g_object_set_data (G_OBJECT (info), "mute_probe", GINT_TO_POINTER (id));
+      id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (info), "mute_probe"));
+      if (id == 0) {
+        id = gst_pad_add_buffer_probe (GST_PAD_CAST (info->object),
+            G_CALLBACK (mute_stream), info);
+        g_object_set_data (G_OBJECT (info), "mute_probe", GINT_TO_POINTER (id));
+      }
     }
   }
 
@@ -1193,71 +1309,45 @@ setup_source (GstPlayBaseBin * play_base_bin, gchar ** new_location)
 
   /* do subs */
   if (subbin) {
-#if 0
-    gint sig4, sig5, sig6;
     GstElement *db;
 
     play_base_bin->subtitle = subbin;
     db = gst_bin_get_by_name (GST_BIN (subbin), "subtitle-decoder");
 
-    /* don't add yet, because we will preroll, and subs shouldn't
-     * preroll (we shouldn't preroll more than once source). */
-    gst_element_set_state (subbin, GST_STATE_PAUSED);
-
     /* do type detection, without adding (so no preroll) */
-    g_signal_connect (G_OBJECT (db),
-        "new-decoded-pad", G_CALLBACK (new_decoded_pad), play_base_bin);
+    g_signal_connect (G_OBJECT (db), "new-decoded-pad",
+        G_CALLBACK (new_decoded_pad), play_base_bin);
     g_signal_connect (G_OBJECT (db), "no-more-pads",
         G_CALLBACK (no_more_pads), play_base_bin);
+    g_signal_connect (G_OBJECT (db), "unknown-type",
+        G_CALLBACK (unknown_type), play_base_bin);
 
     if (!play_base_bin->is_stream) {
-      sig4 = g_signal_connect (G_OBJECT (db),
-          "unknown-type", G_CALLBACK (unknown_type), play_base_bin);
-      sig5 = g_signal_connect (G_OBJECT (subbin), "error",
-          G_CALLBACK (thread_error), error);
-
       /* either when the queues are filled or when the decoder element
        * has no more dynamic streams, the cond is unlocked. We can remove
        * the signal handlers then
        */
-      g_mutex_lock (play_base_bin->group_lock);
-      if (gst_element_set_state (subbin, GST_STATE_PLAYING) ==
-          GST_STATE_CHANGE_SUCCESS) {
-        GST_DEBUG ("waiting for first group...");
-        sig6 = g_signal_connect (G_OBJECT (subbin),
-            "state-changed", G_CALLBACK (state_change), play_base_bin);
-        g_cond_wait (play_base_bin->group_cond, play_base_bin->group_lock);
-        GST_DEBUG ("group done !");
-      } else {
-        GST_DEBUG ("state change failed, subtitle cannot be loaded");
-        sig6 = 0;
-      }
-      g_mutex_unlock (play_base_bin->group_lock);
-
-      if (sig6 != 0)
-        g_signal_handler_disconnect (G_OBJECT (subbin), sig6);
-
-      g_signal_handler_disconnect (G_OBJECT (subbin), sig5);
-      g_signal_handler_disconnect (G_OBJECT (db), sig4);
 
       gst_element_set_state (subbin, GST_STATE_PAUSED);
 
+      GROUP_LOCK (play_base_bin);
+      GST_DEBUG ("waiting for first group...");
+      GROUP_WAIT (play_base_bin);
+      GST_DEBUG ("group done !");
+      GROUP_UNLOCK (play_base_bin);
+
       if (!play_base_bin->building_group ||
           play_base_bin->building_group->type[GST_STREAM_TYPE_TEXT - 1].npads ==
           0) {
-        if (*error) {
-          g_error_free (*error);
-          *error = NULL;
-        }
 
         GST_DEBUG ("No subtitle found - ignoring");
-        gst_object_unref (play_base_bin->subtitle);
+        gst_element_set_state (subbin, GST_STATE_NULL);
+        gst_object_unref (GST_OBJECT (play_base_bin->subtitle));
         play_base_bin->subtitle = NULL;
       } else {
         GST_DEBUG ("Subtitle set-up successful");
       }
     }
-#endif
   }
 
   /* now see if the source element emits raw audio/video all by itself,
@@ -1325,6 +1415,8 @@ setup_source (GstPlayBaseBin * play_base_bin, gchar ** new_location)
       }
     }
 
+    gst_iterator_free (pads_iter);
+
     if (is_raw) {
       no_more_pads (play_base_bin->source, play_base_bin);
       return TRUE;
@@ -1359,6 +1451,10 @@ setup_source (GstPlayBaseBin * play_base_bin, gchar ** new_location)
   g_signal_connect (G_OBJECT (play_base_bin->decoder),
       "unknown-type", G_CALLBACK (unknown_type), play_base_bin);
 
+  if (play_base_bin->subtitle) {
+    gst_bin_add (GST_BIN (play_base_bin), play_base_bin->subtitle);
+  }
+
   play_base_bin->need_rebuild = FALSE;
 
   return TRUE;
@@ -1376,7 +1472,7 @@ no_source:
     }
     prot = gst_uri_get_protocol (play_base_bin->uri);
     if (prot) {
-      GST_ELEMENT_ERROR (play_base_bin, RESOURCE, NOT_FOUND,
+      GST_ELEMENT_ERROR (play_base_bin, RESOURCE, FAILED,
           (_("No URI handler implemented for \"%s\"."), prot), (NULL));
       g_free (prot);
     } else {
@@ -1419,6 +1515,7 @@ prepare_output (GstPlayBaseBin * play_base_bin)
 {
   const GList *item;
   gboolean stream_found = FALSE, no_media = FALSE;
+  gboolean got_video = FALSE, got_subtitle = FALSE;
   GstPlayBaseGroup *group;
 
   group = get_active_group (play_base_bin);
@@ -1429,9 +1526,15 @@ prepare_output (GstPlayBaseBin * play_base_bin)
   for (item = group ? group->streaminfo : NULL; item != NULL; item = item->next) {
     GstStreamInfo *info = GST_STREAM_INFO (item->data);
 
-    if (info->type != GST_STREAM_TYPE_UNKNOWN) {
+    if (info->type == GST_STREAM_TYPE_VIDEO) {
       stream_found = TRUE;
+      got_video = TRUE;
       break;
+    } else if (info->type == GST_STREAM_TYPE_AUDIO) {
+      stream_found = TRUE;
+    } else if (info->type == GST_STREAM_TYPE_TEXT ||
+        info->type == GST_STREAM_TYPE_SUBPICTURE) {
+      got_subtitle = TRUE;
     } else if (!item->prev && !item->next) {
       /* We're no audio/video and the only stream... We could
        * be something not-media that's detected because then our
@@ -1452,14 +1555,24 @@ prepare_output (GstPlayBaseBin * play_base_bin)
   }
 
   if (!stream_found) {
-    if (!no_media) {
+    if (got_subtitle) {
+      GST_ELEMENT_ERROR (play_base_bin, STREAM, WRONG_TYPE,
+          (_("Only a subtitle stream was detected. "
+                  "Either you are loading a subtitle file or some other type of "
+                  "text file, or the media file was not recognized.")), (NULL));
+    } else if (!no_media) {
       GST_ELEMENT_ERROR (play_base_bin, STREAM, CODEC_NOT_FOUND,
-          (_("You do not have a decoder installed to handle \"%s\".  You might need to install the necessary plugins."), play_base_bin->uri), (NULL));
+          (_("You do not have a decoder installed to handle this file. "
+                  "You might need to install the necessary plugins.")), (NULL));
     } else {
       GST_ELEMENT_ERROR (play_base_bin, STREAM, WRONG_TYPE,
-          (_("\"%s\" is not a media file"), play_base_bin->uri), (NULL));
+          (_("This is not a media file")), (NULL));
     }
     return FALSE;
+  } else if (got_subtitle && !got_video) {
+    GST_ELEMENT_ERROR (play_base_bin, STREAM, WRONG_TYPE,
+        (_("A subtitle stream was detected, but no video stream.")), (NULL));
+    return FALSE;
   }
 
   return TRUE;
@@ -1616,7 +1729,7 @@ set_active_source (GstPlayBaseBin * play_base_bin,
     GST_LOG ("Muting group type: %d", type);
     g_object_set (sel, "active-pad", "", NULL);
   } else {
-    GST_LOG ("Unuting group type: %d", type);
+    GST_LOG ("Unmuting group type: %d", type);
   }
   mute_group_type (group, type, !have_active);
 }
@@ -1737,6 +1850,11 @@ gst_play_base_bin_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_pointer (value,
           (gpointer) gst_play_base_bin_get_streaminfo (play_base_bin));
       break;
+    case ARG_STREAMINFO_VALUES:{
+      g_value_set_boxed (value,
+          gst_play_base_bin_get_streaminfo_value_array (play_base_bin));
+      break;
+    }
     case ARG_SOURCE:
       _gst_gvalue_set_gstobject (value, play_base_bin->source);
       break;
@@ -1823,3 +1941,16 @@ gst_play_base_bin_get_streaminfo (GstPlayBaseBin * play_base_bin)
   }
   return info;
 }
+
+const GValueArray *
+gst_play_base_bin_get_streaminfo_value_array (GstPlayBaseBin * play_base_bin)
+{
+  GstPlayBaseGroup *group = get_active_group (play_base_bin);
+  GValueArray *array = NULL;
+
+  if (group) {
+    array = group->streaminfo_value_array;
+  }
+
+  return array;
+}
