@@ -37,6 +37,14 @@
 GST_DEBUG_CATEGORY_STATIC (stream_selector_debug);
 #define GST_CAT_DEFAULT stream_selector_debug
 
+static const GstElementDetails gst_stream_selector_details =
+GST_ELEMENT_DETAILS ("StreamSelector",
+    "Generic",
+    "N-to-1 input stream_selectoring",
+    "Julien Moutte <julien@moutte.net>\n"
+    "Ronald S. Bultje <rbultje@ronald.bitfreak.net>\n"
+    "Jan Schmidt <thaytan@mad.scientist.com>");
+
 static GstStaticPadTemplate gst_stream_selector_sink_factory =
 GST_STATIC_PAD_TEMPLATE ("sink%d",
     GST_PAD_SINK,
@@ -68,6 +76,8 @@ static void gst_stream_selector_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_stream_selector_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
+static GstFlowReturn gst_stream_selector_bufferalloc (GstPad * pad,
+    guint64 offset, guint size, GstCaps * caps, GstBuffer ** buf);
 
 static GstElementClass *parent_class = NULL;
 
@@ -104,13 +114,6 @@ static void
 gst_stream_selector_base_init (GstStreamSelectorClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  static GstElementDetails gst_stream_selector_details =
-      GST_ELEMENT_DETAILS ("StreamSelector",
-      "Generic",
-      "N-to-1 input stream_selectoring",
-      "Julien Moutte <julien@moutte.net>\n"
-      "Ronald S. Bultje <rbultje@ronald.bitfreak.net>\n"
-      "Jan Schmidt <thaytan@mad.scientist.com>");
 
   gst_element_class_set_details (element_class, &gst_stream_selector_details);
 
@@ -126,7 +129,7 @@ gst_stream_selector_class_init (GstStreamSelectorClass * klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->set_property =
       GST_DEBUG_FUNCPTR (gst_stream_selector_set_property);
@@ -179,6 +182,7 @@ gst_stream_selector_set_property (GObject * object, guint prop_id,
     case PROP_ACTIVE_PAD:{
       const gchar *pad_name = g_value_get_string (value);
       GstPad *pad = NULL;
+      GstPad **active_pad_p;
 
       if (strcmp (pad_name, "") != 0) {
         pad = gst_element_get_pad (GST_ELEMENT (object), pad_name);
@@ -191,7 +195,6 @@ gst_stream_selector_set_property (GObject * object, guint prop_id,
           gst_object_unref (pad);
         break;
       }
-
 #if 0
       if (sel->active_sinkpad && (GST_STATE (sel) >= GST_STATE_PAUSED)) {
         gst_pad_set_active (sel->active_sinkpad, FALSE);
@@ -200,8 +203,8 @@ gst_stream_selector_set_property (GObject * object, guint prop_id,
       }
 #endif
 
-      gst_object_replace ((GstObject **) (&sel->active_sinkpad),
-          GST_OBJECT_CAST (pad));
+      active_pad_p = &sel->active_sinkpad;
+      gst_object_replace ((GstObject **) active_pad_p, GST_OBJECT_CAST (pad));
       if (pad)
         gst_object_unref (pad);
 
@@ -233,9 +236,10 @@ gst_stream_selector_get_property (GObject * object, guint prop_id,
     case PROP_ACTIVE_PAD:{
       GST_OBJECT_LOCK (object);
       if (sel->active_sinkpad != NULL) {
-        g_value_set_string (value, gst_pad_get_name (sel->active_sinkpad));
-      } else
+        g_value_take_string (value, gst_pad_get_name (sel->active_sinkpad));
+      } else {
         g_value_set_string (value, "");
+      }
       GST_OBJECT_UNLOCK (object);
       break;
     }
@@ -285,6 +289,48 @@ gst_stream_selector_getcaps (GstPad * pad)
   return gst_pad_peer_get_caps (otherpad);
 }
 
+static GstFlowReturn
+gst_stream_selector_bufferalloc (GstPad * pad, guint64 offset,
+    guint size, GstCaps * caps, GstBuffer ** buf)
+{
+  GstStreamSelector *sel = GST_STREAM_SELECTOR (gst_pad_get_parent (pad));
+  GstFlowReturn result;
+  GstPad *active_sinkpad;
+
+  GST_OBJECT_LOCK (sel);
+  active_sinkpad = sel->active_sinkpad;
+  GST_OBJECT_UNLOCK (sel);
+
+  /* Fallback allocation for buffers from pads except the selected one */
+  if (pad != active_sinkpad) {
+    GST_DEBUG_OBJECT (sel,
+        "Pad %s:%s is not selected. Performing fallback allocation",
+        GST_DEBUG_PAD_NAME (pad));
+
+    *buf = NULL;
+    result = GST_FLOW_OK;
+  } else {
+    result = gst_pad_alloc_buffer (sel->srcpad, offset, size, caps, buf);
+
+    /* FIXME: HACK. If buffer alloc returns not-linked, perform a fallback
+     * allocation.  This should NOT be necessary, because playbin should
+     * properly block the source pad from running until it's finished hooking 
+     * everything up, but playbin needs refactoring first. */
+    if (result == GST_FLOW_NOT_LINKED) {
+      GST_DEBUG_OBJECT (sel,
+          "No peer pad yet - performing fallback allocation for pad %s:%s",
+          GST_DEBUG_PAD_NAME (pad));
+
+      *buf = NULL;
+      result = GST_FLOW_OK;
+    }
+  }
+
+  gst_object_unref (sel);
+
+  return result;
+}
+
 static GList *
 gst_stream_selector_get_linked_pads (GstPad * pad)
 {
@@ -323,6 +369,9 @@ gst_stream_selector_request_new_pad (GstElement * element,
       GST_DEBUG_FUNCPTR (gst_stream_selector_chain));
   gst_pad_set_internal_link_function (sinkpad,
       GST_DEBUG_FUNCPTR (gst_stream_selector_get_linked_pads));
+  gst_pad_set_bufferalloc_function (sinkpad,
+      GST_DEBUG_FUNCPTR (gst_stream_selector_bufferalloc));
+
   gst_element_add_pad (GST_ELEMENT (sel), sinkpad);
 
   return sinkpad;
