@@ -29,6 +29,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#include <string.h>             /* memset, in FD_ZERO macro */
 #include <unistd.h>
 #include <sys/ioctl.h>
 
@@ -36,8 +37,10 @@
 #include <sys/filio.h>
 #endif
 
-#include "gsttcp.h"
+#include <glib.h>
+#include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
+#include <gst/dataprotocol/dataprotocol.h>
 
 GST_DEBUG_CATEGORY_EXTERN (tcp_debug);
 #define GST_CAT_DEFAULT tcp_debug
@@ -106,7 +109,7 @@ gst_tcp_socket_write (int socket, const void *buf, size_t count)
   size_t bytes_written = 0;
 
   while (bytes_written < count) {
-    ssize_t wrote = send (socket, (const char *) buf + bytes_written,
+    ssize_t wrote = send (socket, buf + bytes_written,
         count - bytes_written, MSG_NOSIGNAL);
 
     if (wrote <= 0) {
@@ -118,7 +121,7 @@ gst_tcp_socket_write (int socket, const void *buf, size_t count)
   if (bytes_written < 0)
     GST_WARNING ("error while writing");
   else
-    GST_LOG ("wrote %" G_GSIZE_FORMAT " bytes succesfully", bytes_written);
+    GST_LOG ("wrote %d bytes succesfully", bytes_written);
   return bytes_written;
 }
 
@@ -127,24 +130,30 @@ gst_tcp_socket_write (int socket, const void *buf, size_t count)
  */
 static GstFlowReturn
 gst_tcp_socket_read (GstElement * this, int socket, void *buf, size_t count,
-    GstPoll * fdset)
+    int cancel_fd)
 {
+  fd_set testfds;
+  int maxfdp1;
   ssize_t n;
   size_t bytes_read;
   int num_to_read;
-  int ret;
 
   bytes_read = 0;
 
   while (bytes_read < count) {
     /* do a blocking select on the socket */
+    FD_ZERO (&testfds);
+    FD_SET (socket, &testfds);
+    if (cancel_fd >= 0)
+      FD_SET (cancel_fd, &testfds);
+    maxfdp1 = MAX (socket, cancel_fd) + 1;
+
     /* no action (0) is an error too in our case */
-    if ((ret = gst_poll_wait (fdset, GST_CLOCK_TIME_NONE)) <= 0) {
-      if (ret == -1 && errno == EBUSY)
-        goto cancelled;
-      else
-        goto select_error;
-    }
+    if (select (maxfdp1, &testfds, NULL, NULL, 0) <= 0)
+      goto select_error;
+
+    if (cancel_fd >= 0 && FD_ISSET (cancel_fd, &testfds))
+      goto cancelled;
 
     /* ask how much is available for reading on the socket */
     if (ioctl (socket, FIONREAD, &num_to_read) < 0)
@@ -191,7 +200,7 @@ ioctl_error:
 got_eos:
   {
     GST_DEBUG_OBJECT (this, "Got EOS on socket stream");
-    return GST_FLOW_UNEXPECTED;
+    return GST_FLOW_WRONG_STATE;
   }
 read_error:
   {
@@ -202,19 +211,17 @@ read_error:
 short_read:
   {
     GST_ELEMENT_ERROR (this, RESOURCE, READ, (NULL),
-        ("short read: wanted %d bytes, got %" G_GSSIZE_FORMAT, num_to_read, n));
+        ("short read: wanted %d bytes, got %d", num_to_read, n));
     return GST_FLOW_ERROR;
   }
 }
 
 /* close the socket and reset the fd.  Used to clean up after errors. */
 void
-gst_tcp_socket_close (GstPollFD * socket)
+gst_tcp_socket_close (int *socket)
 {
-  if (socket->fd >= 0) {
-    close (socket->fd);
-    socket->fd = -1;
-  }
+  close (*socket);
+  *socket = -1;
 }
 
 /* read a buffer from the given socket
@@ -224,23 +231,30 @@ gst_tcp_socket_close (GstPollFD * socket)
  *         EOS
  */
 GstFlowReturn
-gst_tcp_read_buffer (GstElement * this, int socket, GstPoll * fdset,
+gst_tcp_read_buffer (GstElement * this, int socket, int cancel_fd,
     GstBuffer ** buf)
 {
+  fd_set testfds;
   int ret;
+  int maxfdp1;
   ssize_t bytes_read;
   int readsize;
 
   *buf = NULL;
 
   /* do a blocking select on the socket */
+  FD_ZERO (&testfds);
+  FD_SET (socket, &testfds);
+  if (cancel_fd >= 0)
+    FD_SET (cancel_fd, &testfds);
+  maxfdp1 = MAX (socket, cancel_fd) + 1;
+
   /* no action (0) is an error too in our case */
-  if ((ret = gst_poll_wait (fdset, GST_CLOCK_TIME_NONE)) <= 0) {
-    if (ret == -1 && errno == EBUSY)
-      goto cancelled;
-    else
-      goto select_error;
-  }
+  if ((ret = select (maxfdp1, &testfds, NULL, NULL, 0)) <= 0)
+    goto select_error;
+
+  if (cancel_fd >= 0 && FD_ISSET (cancel_fd, &testfds))
+    goto cancelled;
 
   /* ask how much is available for reading on the socket */
   if ((ret = ioctl (socket, FIONREAD, &readsize)) < 0)
@@ -262,7 +276,7 @@ gst_tcp_read_buffer (GstElement * this, int socket, GstPoll * fdset,
     /* but mom, you promised to give me readsize bytes! */
     goto short_read;
 
-  GST_LOG_OBJECT (this, "returning buffer of size %d", GST_BUFFER_SIZE (*buf));
+  GST_DEBUG_OBJECT (this, "returning buffer of size %d", GST_BUFFER_SIZE (buf));
   return GST_FLOW_OK;
 
   /* ERRORS */
@@ -299,8 +313,7 @@ read_error:
 short_read:
   {
     GST_ELEMENT_ERROR (this, RESOURCE, READ, (NULL),
-        ("short read: wanted %d bytes, got %" G_GSSIZE_FORMAT, readsize,
-            bytes_read));
+        ("short read: wanted %d bytes, got %d", readsize, bytes_read));
     gst_buffer_unref (*buf);
     *buf = NULL;
     return GST_FLOW_ERROR;
@@ -314,7 +327,7 @@ short_read:
  *         EOS
  */
 GstFlowReturn
-gst_tcp_gdp_read_buffer (GstElement * this, int socket, GstPoll * fdset,
+gst_tcp_gdp_read_buffer (GstElement * this, int socket, int cancel_fd,
     GstBuffer ** buf)
 {
   GstFlowReturn ret;
@@ -326,7 +339,8 @@ gst_tcp_gdp_read_buffer (GstElement * this, int socket, GstPoll * fdset,
   *buf = NULL;
   header = g_malloc (GST_DP_HEADER_LENGTH);
 
-  ret = gst_tcp_socket_read (this, socket, header, GST_DP_HEADER_LENGTH, fdset);
+  ret = gst_tcp_socket_read (this, socket, header, GST_DP_HEADER_LENGTH,
+      cancel_fd);
 
   if (ret != GST_FLOW_OK)
     goto header_read_error;
@@ -344,7 +358,7 @@ gst_tcp_gdp_read_buffer (GstElement * this, int socket, GstPoll * fdset,
   g_free (header);
 
   ret = gst_tcp_socket_read (this, socket, GST_BUFFER_DATA (*buf),
-      GST_BUFFER_SIZE (*buf), fdset);
+      GST_BUFFER_SIZE (*buf), cancel_fd);
 
   if (ret != GST_FLOW_OK)
     goto data_read_error;
@@ -367,8 +381,7 @@ validate_error:
 is_not_buffer:
   {
     GST_ELEMENT_ERROR (this, RESOURCE, READ, (NULL),
-        ("GDP packet contains something that is not a buffer (type %d)",
-            gst_dp_header_payload_type (header)));
+        ("GDP packet contains something that is not a buffer"));
     g_free (header);
     return GST_FLOW_ERROR;
   }
@@ -381,7 +394,7 @@ data_read_error:
 }
 
 GstFlowReturn
-gst_tcp_gdp_read_caps (GstElement * this, int socket, GstPoll * fdset,
+gst_tcp_gdp_read_caps (GstElement * this, int socket, int cancel_fd,
     GstCaps ** caps)
 {
   GstFlowReturn ret;
@@ -395,7 +408,8 @@ gst_tcp_gdp_read_caps (GstElement * this, int socket, GstPoll * fdset,
   *caps = NULL;
   header = g_malloc (GST_DP_HEADER_LENGTH);
 
-  ret = gst_tcp_socket_read (this, socket, header, GST_DP_HEADER_LENGTH, fdset);
+  ret = gst_tcp_socket_read (this, socket, header, GST_DP_HEADER_LENGTH,
+      cancel_fd);
 
   if (ret != GST_FLOW_OK)
     goto header_read_error;
@@ -411,16 +425,15 @@ gst_tcp_gdp_read_caps (GstElement * this, int socket, GstPoll * fdset,
   payload_length = gst_dp_header_payload_length (header);
   payload = g_malloc (payload_length);
 
-  GST_LOG_OBJECT (this,
-      "Reading %" G_GSIZE_FORMAT " bytes for caps packet payload",
+  GST_LOG_OBJECT (this, "Reading %d bytes for caps packet payload",
       payload_length);
 
-  ret = gst_tcp_socket_read (this, socket, payload, payload_length, fdset);
+  ret = gst_tcp_socket_read (this, socket, payload, payload_length, cancel_fd);
 
   if (ret != GST_FLOW_OK)
     goto payload_read_error;
 
-  if (!gst_dp_validate_payload (GST_DP_HEADER_LENGTH, header, payload))
+  if (!gst_dp_validate_payload (payload_length, header, payload))
     goto payload_validate_error;
 
   *caps = gst_dp_caps_from_packet (GST_DP_HEADER_LENGTH, header, payload);
@@ -448,8 +461,7 @@ header_validate_error:
 is_not_caps:
   {
     GST_ELEMENT_ERROR (this, RESOURCE, READ, (NULL),
-        ("GDP packet contains something that is not a caps (type %d)",
-            gst_dp_header_payload_type (header)));
+        ("GDP packet contains something that is not a caps"));
     g_free (header);
     return GST_FLOW_ERROR;
   }
@@ -503,7 +515,7 @@ write_error:
     if (fatal)
       GST_ELEMENT_ERROR (this, RESOURCE, WRITE,
           (_("Error while sending data to \"%s:%d\"."), host, port),
-          ("Only %" G_GSIZE_FORMAT " of %u bytes written: %s",
+          ("Only %d of %d bytes written: %s",
               wrote, GST_BUFFER_SIZE (buffer), g_strerror (errno)));
     return FALSE;
   }
@@ -555,7 +567,7 @@ write_header_error:
     if (fatal)
       GST_ELEMENT_ERROR (this, RESOURCE, WRITE,
           (_("Error while sending gdp header data to \"%s:%d\"."), host, port),
-          ("Only %" G_GSIZE_FORMAT " of %u bytes written: %s",
+          ("Only %d of %d bytes written: %s",
               wrote, length, g_strerror (errno)));
     return FALSE;
   }
@@ -564,7 +576,7 @@ write_payload_error:
     if (fatal)
       GST_ELEMENT_ERROR (this, RESOURCE, WRITE,
           (_("Error while sending gdp payload data to \"%s:%d\"."), host, port),
-          ("Only %" G_GSIZE_FORMAT " of %u bytes written: %s",
+          ("Only %d of %d bytes written: %s",
               wrote, length, g_strerror (errno)));
     return FALSE;
   }
