@@ -460,7 +460,7 @@ aac_type_find (GstTypeFind * tf, gpointer unused)
  * of data (5762) to always detect any mp3.
  */
 
-static guint mp3types_bitrates[2][3][16] =
+static const guint mp3types_bitrates[2][3][16] =
     { {{0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448,},
     {0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384,},
     {0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320,}},
@@ -469,7 +469,7 @@ static guint mp3types_bitrates[2][3][16] =
     {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160,}},
 };
 
-static guint mp3types_freqs[3][3] = { {11025, 12000, 8000},
+static const guint mp3types_freqs[3][3] = { {11025, 12000, 8000},
 {22050, 24000, 16000},
 {44100, 48000, 32000}
 };
@@ -692,9 +692,7 @@ mp3_type_find_at_offset (GstTypeFind * tf, guint64 start_off,
             GST_MP3_TYPEFIND_TRY_SYNC);
         /* make sure we're not id3 tagged */
         head_data = gst_type_find_peek (tf, -128, 3);
-        if (!head_data) {
-          probability = probability * 4 / 5;
-        } else if (memcmp (head_data, "TAG", 3) == 0) {
+        if (head_data && (memcmp (head_data, "TAG", 3) == 0)) {
           probability = 0;
         }
         g_assert (probability <= GST_TYPE_FIND_MAXIMUM);
@@ -782,6 +780,25 @@ suggest:
   }
 }
 
+/*** audio/x-musepack ***/
+
+static GstStaticCaps musepack_caps = GST_STATIC_CAPS ("audio/x-musepack");
+
+#define MUSEPACK_CAPS (gst_static_caps_get(&musepack_caps))
+static void
+musepack_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
+
+  if (data && memcmp (data, "MP+", 3) == 0) {
+    if ((data[3] & 0x7f) == 7) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, MUSEPACK_CAPS);
+    } else {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY + 10, MUSEPACK_CAPS);
+    }
+  }
+}
+
 /*** audio/x-ac3 ***/
 static GstStaticCaps ac3_caps = GST_STATIC_CAPS ("audio/x-ac3");
 
@@ -816,64 +833,58 @@ GST_STATIC_CAPS ("audio/x-wavpack-correction, framed = (boolean) false");
 static void
 wavpack_type_find (GstTypeFind * tf, gpointer unused)
 {
-  guint8 *data = gst_type_find_peek (tf, 0, 32);
+  guint64 offset;
+  guint32 blocksize;
+  guint8 *data;
 
+  data = gst_type_find_peek (tf, 0, 32);
   if (!data)
     return;
 
-  if (data[0] == 'w' && data[1] == 'v' && data[2] == 'p' && data[3] == 'k') {
-    guint32 blocksize, sublen;
-    gint32 left;
+  if (data[0] != 'w' || data[1] != 'v' || data[2] != 'p' || data[3] != 'k')
+    return;
 
-    GST_LOG ("got wavpack header");
+  /* Note: wavpack blocks can be fairly large (easily 60-110k), possibly
+   * larger than the max. limits imposed by certain typefinding elements
+   * like id3demux or apedemux, so typefinding is most likely only going to
+   * work in pull-mode */
+  blocksize = GST_READ_UINT32_LE (data + 4);
+  GST_LOG ("wavpack header, blocksize=0x%04x", blocksize);
+  offset = 32;
+  while (offset < 32 + blocksize) {
+    guint32 sublen;
 
-    /* wavpack blocks can be fairly large, possibly larger than the max.
-     * limits imposed by certain typefinding elements like id3demux or
-     * apedemux. So if the first wavpack block is larger than any particular
-     * limit, we try to get a smaller chunk and hope we get lucky parsing
-     * only bits of it (case at hand: first wavpack block: 42kB, with a
-     * max. limit imposed by apedemux/id3demux: 40kB) */
-    blocksize = GST_READ_UINT32_LE (data + 4);
-    do {
-      /* peek from offset 0, otherwise it won't work with apedemux */
-      data = gst_type_find_peek (tf, 0, 32 + blocksize);
-      if (data != NULL)
-        break;
-      if (32 + blocksize < 512) /* random threshold */
-        break;
-      blocksize = (blocksize * 3) / 4;
-    } while (data == NULL);
-    if (!data)
-      return;
-    data += 32;
-    left = (gint32) blocksize;
-    while (left > 2) {
-      sublen = ((guint32) data[1]) << 1;
-      if (data[0] & 0x80) {
-        sublen |= (((guint32) data[2]) << 9) | (((guint32) data[3]) << 17);
-        sublen += 1 + 3;        /* id + length */
-      } else {
-        sublen += 1 + 1;        /* id + length */
-      }
-      if (sublen > blocksize)
-        return;
-      if ((data[0] & 0x20) == 0) {
-        switch (data[0] & 0x0f) {
-          case 0xa:            /* ID_WV_BITSTREAM  */
-          case 0xc:            /* ID_WVX_BITSTREAM */
-            gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, WAVPACK_CAPS);
-            return;
-          case 0xb:            /* ID_WVC_BITSTREAM */
-            gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY,
-                WAVPACK_CORRECTION_CAPS);
-            return;
-          default:
-            break;
-        }
-      }
-      left -= sublen;
-      data += sublen;
+    /* get chunk header */
+    GST_LOG ("peeking at chunk at offset 0x%04x", (guint) offset);
+    data = gst_type_find_peek (tf, offset, 4);
+    if (data == NULL)
+      break;
+    sublen = ((guint32) data[1]) << 1;
+    if (data[0] & 0x80) {
+      sublen |= (((guint32) data[2]) << 9) | (((guint32) data[3]) << 17);
+      sublen += 1 + 3;          /* id + length */
+    } else {
+      sublen += 1 + 1;          /* id + length */
     }
+    if (sublen > blocksize - offset + 32) {
+      GST_LOG ("chunk length too big (%u > %u)", sublen, blocksize - offset);
+      break;
+    }
+    if ((data[0] & 0x20) == 0) {
+      switch (data[0] & 0x0f) {
+        case 0xa:              /* ID_WV_BITSTREAM  */
+        case 0xc:              /* ID_WVX_BITSTREAM */
+          gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, WAVPACK_CAPS);
+          return;
+        case 0xb:              /* ID_WVC_BITSTREAM */
+          gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY,
+              WAVPACK_CORRECTION_CAPS);
+          return;
+        default:
+          break;
+      }
+    }
+    offset += sublen;
   }
 }
 
@@ -882,56 +893,56 @@ wavpack_type_find (GstTypeFind * tf, gpointer unused)
 
 static GstStaticCaps mpeg_sys_caps = GST_STATIC_CAPS ("video/mpeg, "
     "systemstream = (boolean) true, mpegversion = (int) [ 1, 2 ]");
+
 #define MPEG_SYS_CAPS gst_static_caps_get(&mpeg_sys_caps)
-#define IS_MPEG_HEADER(data)            ((((guint8 *)data)[0] == 0x00) &&       \
-                                         (((guint8 *)data)[1] == 0x00) &&       \
-                                         (((guint8 *)data)[2] == 0x01) &&       \
-                                         (((guint8 *)data)[3] == 0xBA))
-#define IS_MPEG_SYSTEM_HEADER(data)     ((((guint8 *)data)[0] == 0x00) &&       \
-                                         (((guint8 *)data)[1] == 0x00) &&       \
-                                         (((guint8 *)data)[2] == 0x01) &&       \
-                                         (((guint8 *)data)[3] == 0xBB))
-#define IS_MPEG_PACKET_HEADER(data)     ((((guint8 *)data)[0] == 0x00) &&       \
-                                         (((guint8 *)data)[1] == 0x00) &&       \
-                                         (((guint8 *)data)[2] == 0x01) &&       \
-                                         ((((guint8 *)data)[3] & 0x80) == 0x80))
-#define IS_MPEG_PES_HEADER(data)        ((((guint8 *)data)[0] == 0x00) &&       \
-                                         (((guint8 *)data)[1] == 0x00) &&       \
-                                         (((guint8 *)data)[2] == 0x01) && \
-                                         ((((guint8 *)data)[3] == 0xE0) || \
-                                          (((guint8 *)data)[3] == 0xC0) || \
-                                          (((guint8 *)data)[3] == 0xBD)))
+#define IS_MPEG_HEADER(data)            ((((guint8 *)(data))[0] == 0x00) &&  \
+                                         (((guint8 *)(data))[1] == 0x00) &&  \
+                                         (((guint8 *)(data))[2] == 0x01))
+
+#define IS_MPEG_PACK_HEADER(data)       (IS_MPEG_HEADER (data) &&            \
+                                         (((guint8 *)(data))[3] == 0xBA))
+
+#define IS_MPEG_SYSTEM_HEADER(data)     (IS_MPEG_HEADER (data) &&            \
+                                         (((guint8 *)(data))[3] == 0xBB))
+#define IS_MPEG_PACKET_HEADER(data)     (IS_MPEG_HEADER (data) &&            \
+                                         ((((guint8 *)(data))[3] & 0x80) == 0x80))
+
+#define IS_MPEG_PES_HEADER(data)        (IS_MPEG_HEADER (data) &&            \
+                                         ((((guint8 *)(data))[3] == 0xE0) || \
+                                          (((guint8 *)(data))[3] == 0xC0) || \
+                                          (((guint8 *)(data))[3] == 0xBD)))
 
 static void
 mpeg2_sys_type_find (GstTypeFind * tf, gpointer unused)
 {
   guint8 *data = gst_type_find_peek (tf, 0, 5);
+  gint mpegversion;
 
-  if (data && IS_MPEG_HEADER (data)) {
+  if (data && IS_MPEG_PACK_HEADER (data)) {
     if ((data[4] & 0xC0) == 0x40) {
       /* type 2 */
-      GstCaps *caps = gst_caps_copy (MPEG_SYS_CAPS);
-
-      gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
-          G_TYPE_INT, 2, NULL);
-      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
+      mpegversion = 2;
+      goto suggest;
     } else if ((data[4] & 0xF0) == 0x20) {
-      GstCaps *caps = gst_caps_copy (MPEG_SYS_CAPS);
-
-      gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
-          G_TYPE_INT, 1, NULL);
-      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
-      gst_caps_unref (caps);
+      mpegversion = 1;
+      goto suggest;
     }
   } else if (data && IS_MPEG_PES_HEADER (data)) {
     /* PES stream */
+    mpegversion = 2;
+    goto suggest;
+  }
+
+  return;
+suggest:
+  {
     GstCaps *caps = gst_caps_copy (MPEG_SYS_CAPS);
 
     gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
-        G_TYPE_INT, 2, NULL);
+        G_TYPE_INT, mpegversion, NULL);
     gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
+    gst_caps_unref (caps);
   }
-
 };
 
 /* ATTENTION: ugly return value:
@@ -1044,7 +1055,7 @@ mpeg1_sys_type_find (GstTypeFind * tf, gpointer unused)
         break;
       size = GST_MPEG_TYPEFIND_SYNC_SIZE;
     }
-    if (IS_MPEG_HEADER (data)) {
+    if (IS_MPEG_PACK_HEADER (data)) {
       /* found packet start code */
       guint found = 0;
       guint packet_size = 0;
@@ -1078,11 +1089,13 @@ static GstStaticCaps mpegts_caps = GST_STATIC_CAPS ("video/mpegts, "
     "systemstream = (boolean) true, packetsize = (int) [ 188, 208 ]");
 #define MPEGTS_CAPS gst_static_caps_get(&mpegts_caps)
 
-#define GST_MPEGTS_TYPEFIND_TRY_HEADERS 4
+#define GST_MPEGTS_TYPEFIND_MIN_HEADERS 4
+#define GST_MPEGTS_TYPEFIND_MAX_HEADERS 10
 #define GST_MPEGTS_MAX_PACKET_SIZE 204
-#define GST_MPEGTS_TYPEFIND_MAX_SYNC (10 * GST_MPEGTS_MAX_PACKET_SIZE)
 #define GST_MPEGTS_TYPEFIND_SYNC_SIZE \
-            (GST_MPEGTS_TYPEFIND_TRY_HEADERS * GST_MPEGTS_MAX_PACKET_SIZE)
+            (GST_MPEGTS_TYPEFIND_MIN_HEADERS * GST_MPEGTS_MAX_PACKET_SIZE)
+#define GST_MPEGTS_TYPEFIND_MAX_SYNC \
+            (GST_MPEGTS_TYPEFIND_MAX_HEADERS * GST_MPEGTS_MAX_PACKET_SIZE)
 
 #define MPEGTS_HDR_SIZE 4
 #define IS_MPEGTS_HEADER(data) (((data)[0] == 0x47) && \
@@ -1098,7 +1111,7 @@ mpeg_ts_probe_headers (GstTypeFind * tf, guint64 offset, gint packet_size)
   gint found = 1;
   guint8 *data = NULL;
 
-  while (found < GST_MPEGTS_TYPEFIND_TRY_HEADERS) {
+  while (found < GST_MPEGTS_TYPEFIND_MAX_HEADERS) {
     offset += packet_size;
 
     data = gst_type_find_peek (tf, offset, MPEGTS_HDR_SIZE);
@@ -1138,14 +1151,24 @@ mpeg_ts_type_find (GstTypeFind * tf, gpointer unused)
       gint p;
 
       for (p = 0; p < n_pack_sizes; p++) {
+        gint found;
+
         /* Probe ahead at size pack_sizes[p] */
-        if (mpeg_ts_probe_headers (tf, skipped, pack_sizes[p]) >=
-            GST_MPEGTS_TYPEFIND_TRY_HEADERS) {
+        found = mpeg_ts_probe_headers (tf, skipped, pack_sizes[p]);
+        if (found >= GST_MPEGTS_TYPEFIND_MIN_HEADERS) {
+          gint probability;
           GstCaps *caps = gst_caps_copy (MPEGTS_CAPS);
 
           gst_structure_set (gst_caps_get_structure (caps, 0), "packetsize",
               G_TYPE_INT, pack_sizes[p], NULL);
-          gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 1, caps);
+
+          /* found at least 4 headers. 10 headers = MAXIMUM probability. 
+           * Arbitrarily, I assigned 10% probability for each header we
+           * found, 40% -> 100% */
+
+          probability = 10 * MIN (found, 10);
+
+          gst_type_find_suggest (tf, probability, caps);
           gst_caps_unref (caps);
           return;
         }
@@ -1833,26 +1856,21 @@ dv_type_find (GstTypeFind * tf, gpointer private)
 /*** application/ogg and application/x-annodex ***/
 static GstStaticCaps ogg_caps = GST_STATIC_CAPS ("application/ogg");
 static GstStaticCaps annodex_caps = GST_STATIC_CAPS ("application/x-annodex");
-static GstStaticCaps ogganx_caps =
-    GST_STATIC_CAPS ("application/ogg; application/x-annodex");
-#define OGGANX_CAPS (gst_static_caps_get(&ogganx_caps))
+
+#define OGGANX_CAPS (gst_static_caps_get(&annodex_caps))
 
 static void
 ogganx_type_find (GstTypeFind * tf, gpointer private)
 {
-  guint8 *data = gst_type_find_peek (tf, 28, 8);
-  gboolean is_annodex = FALSE;
+  guint8 *data = gst_type_find_peek (tf, 0, 4);
 
-  if ((data != NULL) && (memcmp (data, "fishead\0", 8) == 0)) {
-    is_annodex = TRUE;
-  }
-
-  data = gst_type_find_peek (tf, 0, 4);
   if ((data != NULL) && (memcmp (data, "OggS", 4) == 0)) {
-    if (is_annodex == TRUE) {
-      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM,
-          gst_static_caps_get (&annodex_caps));
-    }
+
+    /* Check for an annodex fishbone header */
+    data = gst_type_find_peek (tf, 28, 8);
+    if (memcmp (data, "fishead\0", 8) == 0)
+      gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, OGGANX_CAPS);
+
     gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM,
         gst_static_caps_get (&ogg_caps));
   }
@@ -2183,7 +2201,7 @@ msdos_type_find (GstTypeFind * tf, gpointer unused)
 /*** generic typefind for streams that have some data at a specific position***/
 typedef struct
 {
-  guint8 *data;
+  const guint8 *data;
   guint size;
   guint probability;
   GstCaps *caps;
@@ -2205,15 +2223,24 @@ start_with_type_find (GstTypeFind * tf, gpointer private)
   }
 }
 
+static void
+sw_data_destroy (GstTypeFindData * sw_data)
+{
+  if (G_LIKELY (sw_data->caps != NULL))
+    gst_caps_unref (sw_data->caps);
+  g_free (sw_data);
+}
+
 #define TYPE_FIND_REGISTER_START_WITH(plugin,name,rank,ext,_data,_size,_probability)\
 G_BEGIN_DECLS{                                                          \
   GstTypeFindData *sw_data = g_new (GstTypeFindData, 1);                \
-  sw_data->data = (gpointer)_data;                                      \
+  sw_data->data = (const guint8 *)_data;                                \
   sw_data->size = _size;                                                \
   sw_data->probability = _probability;                                  \
   sw_data->caps = gst_caps_new_simple (name, NULL);                     \
   if (!gst_type_find_register (plugin, name, rank, start_with_type_find,\
-                      ext, sw_data->caps, sw_data, (GDestroyNotify) (g_free))) {        \
+                      ext, sw_data->caps, sw_data,                      \
+                     (GDestroyNotify) (sw_data_destroy))) {             \
     gst_caps_unref (sw_data->caps);                                     \
     g_free (sw_data);                                                   \
   }                                                                     \
@@ -2242,7 +2269,8 @@ G_BEGIN_DECLS{                                                          \
   sw_data->probability = GST_TYPE_FIND_MAXIMUM;                         \
   sw_data->caps = gst_caps_new_simple (name, NULL);                     \
   if (!gst_type_find_register (plugin, name, rank, riff_type_find,      \
-                      ext, sw_data->caps, sw_data, (GDestroyNotify) (g_free))) {                        \
+                      ext, sw_data->caps, sw_data,                      \
+                      (GDestroyNotify) (sw_data_destroy))) {            \
     gst_caps_unref (sw_data->caps);                                     \
     g_free (sw_data);                                                   \
   }                                                                     \
@@ -2261,6 +2289,10 @@ plugin_init (GstPlugin * plugin)
   /* can't initialize this via a struct as caps can't be statically initialized */
 
   /* note: asx/wax/wmx are XML files, asf doesn't handle them */
+  /* FIXME-0.11: these should be const,
+     this requires gstreamer/gst/gsttypefind::gst_type_find_register()
+     to have define the parameter as const
+   */
   static gchar *asf_exts[] = { "asf", "wm", "wma", "wmv", NULL };
   static gchar *au_exts[] = { "au", "snd", NULL };
   static gchar *avi_exts[] = { "avi", NULL };
@@ -2342,9 +2374,8 @@ plugin_init (GstPlugin * plugin)
       asf_exts,
       "\060\046\262\165\216\146\317\021\246\331\000\252\000\142\316\154", 16,
       GST_TYPE_FIND_MAXIMUM);
-  /* -1 so id3v1 or apev1/2 are detected with higher preference */
-  TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-musepack", GST_RANK_PRIMARY,
-      musepack_exts, "MP+", 3, GST_TYPE_FIND_LIKELY + 10);
+  TYPE_FIND_REGISTER (plugin, "audio/x-musepack", GST_RANK_PRIMARY,
+      musepack_type_find, musepack_exts, MUSEPACK_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-au", GST_RANK_MARGINAL,
       au_type_find, au_exts, AU_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-msvideo", GST_RANK_PRIMARY,
@@ -2508,8 +2539,12 @@ plugin_init (GstPlugin * plugin)
       ar_type_find, ar_exts, AR_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-ms-dos-executable",
       GST_RANK_SECONDARY, msdos_type_find, msdos_exts, MSDOS_CAPS, NULL, NULL);
+#if 0
   TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-dirac",
-      GST_RANK_PRIMARY, NULL, "BBCD", 4, GST_TYPE_FIND_LIKELY);
+      GST_RANK_PRIMARY, NULL, "BBCD", 4, GST_TYPE_FIND_MAXIMUM);
+#endif
+  TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-dirac",
+      GST_RANK_PRIMARY, NULL, "KW-DIRAC", 8, GST_TYPE_FIND_MAXIMUM);
 
   return TRUE;
 }
