@@ -34,14 +34,13 @@ struct paintinfo_struct
   unsigned char *dest;          /* pointer to first byte of video data */
   unsigned char *yp, *up, *vp;  /* pointers to first byte of each component
                                  * for both packed/planar YUV and RGB */
-  unsigned char *ap;            /* pointer to first byte of alpha component */
   unsigned char *endptr;        /* pointer to byte beyond last video data */
   int ystride;
   int ustride;
   int vstride;
   int width;
   int height;
-  const struct vts_color_struct *color;
+  struct vts_color_struct *color;
   void (*paint_hline) (paintinfo * p, int x, int y, int w);
 };
 
@@ -57,7 +56,6 @@ struct fourcc_list_struct
   unsigned int red_mask;
   unsigned int green_mask;
   unsigned int blue_mask;
-  unsigned int alpha_mask;
 };
 
 struct fourcc_list_struct *
@@ -69,27 +67,11 @@ struct fourcc_list_struct *
 GstStructure *
         paint_get_structure             (struct fourcc_list_struct *format);
 int     gst_video_test_src_get_size     (GstVideoTestSrc * v, int w, int h);
-void    gst_video_test_src_smpte        (GstVideoTestSrc * v,
+void    gst_video_test_src_smpte                (GstVideoTestSrc * v,
                                          unsigned char *dest, int w, int h);
 void    gst_video_test_src_snow         (GstVideoTestSrc * v,
                                          unsigned char *dest, int w, int h);
-void    gst_video_test_src_black        (GstVideoTestSrc * v,
-                                         unsigned char *dest, int w, int h);
-void    gst_video_test_src_white        (GstVideoTestSrc * v,
-                                         unsigned char *dest, int w, int h);
-void    gst_video_test_src_red          (GstVideoTestSrc * v,
-                                         unsigned char *dest, int w, int h);
-void    gst_video_test_src_green        (GstVideoTestSrc * v,
-                                         unsigned char *dest, int w, int h);
-void    gst_video_test_src_blue         (GstVideoTestSrc * v,
-                                         unsigned char *dest, int w, int h);
-void    gst_video_test_src_checkers1    (GstVideoTestSrc * v,
-                                         unsigned char *dest, int w, int h);
-void    gst_video_test_src_checkers2    (GstVideoTestSrc * v,
-                                         unsigned char *dest, int w, int h);
-void    gst_video_test_src_checkers4    (GstVideoTestSrc * v,
-                                         unsigned char *dest, int w, int h);
-void    gst_video_test_src_checkers8    (GstVideoTestSrc * v,
+void    gst_video_test_src_black                (GstVideoTestSrc * v,
                                          unsigned char *dest, int w, int h);
 
 extern struct fourcc_list_struct fourcc_list[];
