@@ -34,10 +34,10 @@ static const GstElementDetails gst_v4lsrc_details =
 GST_ELEMENT_DETAILS ("Video (video4linux/raw) Source",
     "Source/Video",
     "Reads raw frames from a video4linux device",
-    "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
 
-GST_DEBUG_CATEGORY_STATIC (v4lsrc_debug);
+GST_DEBUG_CATEGORY (v4lsrc_debug);
 #define GST_CAT_DEFAULT v4lsrc_debug
 
 
@@ -53,11 +53,6 @@ enum
 
 GST_BOILERPLATE (GstV4lSrc, gst_v4lsrc, GstV4lElement, GST_TYPE_V4LELEMENT);
 
-static GstStaticPadTemplate v4l_src_template = GST_STATIC_PAD_TEMPLATE ("src",
-    GST_PAD_SRC,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("ANY")
-    );
 
 /* basesrc methods */
 static gboolean gst_v4lsrc_start (GstBaseSrc * src);
@@ -65,8 +60,8 @@ static gboolean gst_v4lsrc_stop (GstBaseSrc * src);
 static gboolean gst_v4lsrc_set_caps (GstBaseSrc * src, GstCaps * caps);
 static GstCaps *gst_v4lsrc_get_caps (GstBaseSrc * src);
 static GstFlowReturn gst_v4lsrc_create (GstPushSrc * src, GstBuffer ** out);
-static gboolean gst_v4lsrc_query (GstBaseSrc * bsrc, GstQuery * query);
-static void gst_v4lsrc_fixate (GstBaseSrc * bsrc, GstCaps * caps);
+
+static void gst_v4lsrc_fixate (GstPad * pad, GstCaps * caps);
 
 static void gst_v4lsrc_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
@@ -82,7 +77,8 @@ gst_v4lsrc_base_init (gpointer g_class)
   gst_element_class_set_details (gstelement_class, &gst_v4lsrc_details);
 
   gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&v4l_src_template));
+      gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
+          gst_caps_new_any ()));
 }
 
 static void
@@ -102,21 +98,20 @@ gst_v4lsrc_class_init (GstV4lSrcClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_AUTOPROBE,
       g_param_spec_boolean ("autoprobe", "Autoprobe",
           "Whether the device should be probed for all possible features",
-          TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_AUTOPROBE_FPS,
       g_param_spec_boolean ("autoprobe-fps", "Autoprobe FPS",
           "Whether the device should be probed for framerates",
-          TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_COPY_MODE,
       g_param_spec_boolean ("copy-mode", "Copy mode",
           "Whether to send out copies of buffers, or direct pointers to the mmap region",
-          TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass),
       PROP_TIMESTAMP_OFFSET, g_param_spec_int64 ("timestamp-offset",
           "Timestamp offset",
           "A time offset subtracted from timestamps set on buffers (in ns)",
-          G_MININT64, G_MAXINT64, 0,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_MININT64, G_MAXINT64, 0, G_PARAM_READWRITE));
 
   GST_DEBUG_CATEGORY_INIT (v4lsrc_debug, "v4lsrc", 0, "V4L source element");
 
@@ -124,8 +119,6 @@ gst_v4lsrc_class_init (GstV4lSrcClass * klass)
   basesrc_class->set_caps = gst_v4lsrc_set_caps;
   basesrc_class->start = gst_v4lsrc_start;
   basesrc_class->stop = gst_v4lsrc_stop;
-  basesrc_class->fixate = gst_v4lsrc_fixate;
-  basesrc_class->query = gst_v4lsrc_query;
 
   pushsrc_class->create = gst_v4lsrc_create;
 }
@@ -147,7 +140,9 @@ gst_v4lsrc_init (GstV4lSrc * v4lsrc, GstV4lSrcClass * klass)
 
   v4lsrc->fps_list = NULL;
 
-  gst_base_src_set_format (GST_BASE_SRC (v4lsrc), GST_FORMAT_TIME);
+  gst_pad_set_fixatecaps_function (GST_BASE_SRC_PAD (v4lsrc),
+      gst_v4lsrc_fixate);
+
   gst_base_src_set_live (GST_BASE_SRC (v4lsrc), TRUE);
 }
 
@@ -206,12 +201,12 @@ gst_v4lsrc_get_property (GObject * object,
 
 /* this function is a bit of a last resort */
 static void
-gst_v4lsrc_fixate (GstBaseSrc * bsrc, GstCaps * caps)
+gst_v4lsrc_fixate (GstPad * pad, GstCaps * caps)
 {
   GstStructure *structure;
   int i;
   int targetwidth, targetheight;
-  GstV4lSrc *v4lsrc = GST_V4LSRC (bsrc);
+  GstV4lSrc *v4lsrc = GST_V4LSRC (gst_pad_get_parent (pad));
   struct video_capability *vcap = &GST_V4LELEMENT (v4lsrc)->vcap;
   struct video_window *vwin = &GST_V4LELEMENT (v4lsrc)->vwin;
 
@@ -236,9 +231,9 @@ gst_v4lsrc_fixate (GstBaseSrc * bsrc, GstCaps * caps)
   GST_DEBUG_OBJECT (v4lsrc, "targetting %dx%d", targetwidth, targetheight);
 
   for (i = 0; i < gst_caps_get_size (caps); ++i) {
+    structure = gst_caps_get_structure (caps, i);
     const GValue *v;
 
-    structure = gst_caps_get_structure (caps, i);
     gst_structure_fixate_field_nearest_int (structure, "width", targetwidth);
     gst_structure_fixate_field_nearest_int (structure, "height", targetheight);
     gst_structure_fixate_field_nearest_fraction (structure, "framerate", 15, 2);
@@ -330,8 +325,8 @@ gst_v4lsrc_palette_to_caps (int palette)
             "bpp = (int) 24, "
             "depth = (int) 24, "
             "endianness = (int) BIG_ENDIAN, "
-            "red_mask = 0xFF0000, "
-            "green_mask = 0x00FF00, " "blue_mask = 0x0000FF");
+            "red_mask = 0x0000FF, "
+            "green_mask = 0x00FF00, " "blue_mask = 0xFF0000");
         break;
       case VIDEO_PALETTE_RGB32:
         caps = gst_caps_from_string ("video/x-raw-rgb, "
@@ -354,7 +349,7 @@ gst_v4lsrc_palette_to_caps (int palette)
 }
 
 static GstCaps *
-gst_v4lsrc_get_any_caps (void)
+gst_v4lsrc_get_any_caps ()
 {
   gint i;
   GstCaps *caps = gst_caps_new_empty (), *one;
@@ -620,53 +615,6 @@ gst_v4lsrc_set_caps (GstBaseSrc * src, GstCaps * caps)
   return TRUE;
 }
 
-static gboolean
-gst_v4lsrc_query (GstBaseSrc * bsrc, GstQuery * query)
-{
-  GstV4lSrc *v4lsrc;
-  gboolean res = FALSE;
-
-  v4lsrc = GST_V4LSRC (bsrc);
-
-  switch (GST_QUERY_TYPE (query)) {
-    case GST_QUERY_LATENCY:
-    {
-      GstClockTime min_latency, max_latency;
-      gint fps_n, fps_d;
-
-      /* device must be open */
-      if (!GST_V4L_IS_OPEN (v4lsrc))
-        goto done;
-
-      /* we must have a framerate */
-      if (!(res = gst_v4lsrc_get_fps (v4lsrc, &fps_n, &fps_d)))
-        goto done;
-
-      /* min latency is the time to capture one frame */
-      min_latency = gst_util_uint64_scale_int (GST_SECOND, fps_d, fps_n);
-
-      /* max latency is total duration of the frame buffer */
-      max_latency = v4lsrc->mbuf.frames * min_latency;
-
-      GST_DEBUG_OBJECT (bsrc,
-          "report latency min %" GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (min_latency), GST_TIME_ARGS (max_latency));
-
-      /* we are always live, the min latency is 1 frame and the max latency is
-       * the complete buffer of frames. */
-      gst_query_set_latency (query, TRUE, min_latency, max_latency);
-
-      res = TRUE;
-      break;
-    }
-    default:
-      res = GST_BASE_SRC_CLASS (parent_class)->query (bsrc, query);
-      break;
-  }
-done:
-  return res;
-}
-
 /* start and stop are not symmetric -- start will open the device, but not start
    capture. it's setcaps that will start capture, which is called via basesrc's
    negotiate method. stop will both stop capture and close the device.
@@ -676,6 +624,9 @@ gst_v4lsrc_start (GstBaseSrc * src)
 {
   GstV4lSrc *v4lsrc = GST_V4LSRC (src);
 
+  if (!GST_BASE_SRC_CLASS (parent_class)->start (src))
+    return FALSE;
+
   v4lsrc->offset = 0;
 
   return TRUE;
@@ -694,6 +645,9 @@ gst_v4lsrc_stop (GstBaseSrc * src)
       return FALSE;
   }
 
+  if (!GST_BASE_SRC_CLASS (parent_class)->stop (src))
+    return FALSE;
+
   g_list_free (v4lsrc->colorspaces);
   v4lsrc->colorspaces = NULL;
 
